<?php
namespace Teknisa\FIN\Controllers;

include('AccesstageWSConsts.php');

use Teknisa\FIN\Modules\MovimentacaoBancaria\Services\Bco54200ExtratoBancario;
use Teknisa\FIN\Modules\ContaPagar\Services\RetornoPagarService;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class AccesstageWS { 
    
    private static $nrorg;
    private static $conn;
    private static $header;
    
    protected $Bco54200Service;
    protected $entityManager;
    
    public function __construct( Bco54200ExtratoBancario $Bco54200Service, RetornoPagarService $RetornoServices, $entityManager){
    	$this->Bco54200Service = $Bco54200Service;
        $this->RetornoServices = $RetornoServices;
        $this->entityManager = $entityManager;
    }
    
    public function executeWs(){
        try{
            
            date_default_timezone_set("America/Sao_Paulo");
            
            /* Seta dados de login no header do soapclient para acessar os dados no webservice */        
            AccesStageWS::setHeaderSoapClient();
            
            /* Conexao a ser usada quando o arquivo é aplicado no cliente */
            //self::$conn = oci_connect("/", "", USERNAME,"AL32UTF8", OCI_CRED_EXT);
    
            /* Conexao a ser usada quando o arquivo é aplicado para testes na Teknisa  */
            self::$conn = oci_connect('THEFIFTIES', 'teknisa', '192.168.122.5/pdborcl');
            
            /* Pega NRORG  */
            $sql = "select NVL(decode(SYS_CONTEXT('nrorg_ctx', 'nrorg'), 0, 1), 1) NRORG from dual";
            $statement = oci_parse(self::$conn, $sql);
            oci_execute($statement, OCI_NO_AUTO_COMMIT);
            self::$nrorg = oci_fetch_assoc($statement)['NRORG'];
                
            $msgsDisponiveis = AccesstageWS::getListaDocumentos();
            
            var_dump($msgsDisponiveis);die;
            
            AccesstageWS::insereTrackingId($msgsDisponiveis);
            
            AccesstageWS::controlaConteudoMsgPorTrackId($msgsDisponiveis);
            
            AccesStageWS::importaExtratoOFX();
            
            //AccesStageWS::importaRetornoCNAB();
            
            //AccesstageWS::confirmaRetiradaArquivo();
            
            oci_commit(self::$conn);
        }catch(\Exception $e) {
            oci_rollback(self::$conn);
            //$this->entityManager->rollback();
            echo($e->getMessage());
        }
    }
    
    private static function setHeaderSoapClient(){
        /*
        * Gera conexão via SoapClient através da URL disponibilizada pela AccesStage 
        * A conexão fica preparada para ser possível acessar o procedimento "process".
        * Para mais informações sobre serviços soap e a documentação das
        * classes SoapClient e SoapServer e seus respectivos metódos,
        * consulte - http://php.net/manual/en/book.soap.php.
        */
        
        //Configurações para protocolo de segurança que é lido no header do Soap para todas as requisições
        $wsseNamespace = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd';
        $security = new \SoapVar(
            array(new \SoapVar(
                array(
                    new \SoapVar(ID_EMPRESA_CONC, XSD_STRING, null, null, 'Username', $wsseNamespace),
                    new \SoapVar(SENHA_CONC, XSD_STRING, null, null, 'Password', $wsseNamespace)
                ), 
                SOAP_ENC_OBJECT, 
                null, 
                null, 
                'UsernameToken', 
                $wsseNamespace
            )), 
            SOAP_ENC_OBJECT
        );

        self::$header = new \SoapHeader($wsseNamespace, 'Security', $security, false);
    }

    //Função utilizada para obter a Tracking Id de todos os documentos do tipo esperado.
    //Retornará um array contendo, em cada posição, um número de TrackingId.
    private static function getListaDocumentos(){
       try{
            $listaDocDisponivel = new \SoapClient(LISTAMSGDISPONIVEL, array("keep_alive" => true));
            
            var_dump($listaDocDisponivel);die;
            $listaDocDisponivel->__setSoapHeaders(self::$header);
            if (!$listaDocDisponivel) {
                throw new \Exception("Não foi possível obter conexão para lista de mensagens disponiveis.");
            }
            $response = $listaDocDisponivel->__soapCall('process', array());
            if(!$response) {
                throw new \Exception("Não foi possível encontrar dados.");
            }
            $msgsDisponiveis = $response->MsgDisponivel;
            return $msgsDisponiveis;
        }catch(\Exception $e){
            echo($e->getMessage());
        }
    }   
    
    private static function insereTrackingId($msgsDisponiveis){
        
        //$mensagensDisponiveis = [];
        foreach ($msgsDisponiveis as $msgDisponivel) {
            if(AccesstageWS::verifyTrackingId($msgDisponivel)){
                //array_push($mensagensDisponiveis, $msgDisponivel);
                $IDATIVO = 'S';
                $NRCTRLTRCKIDACS = intval(AccesstageWS::newCode("CTRLTRACKIDBANC"));
                $msgDisponivel->dtaHoraDisp = AccesstageWS::formatDate($msgDisponivel->dtaHoraDisp);
                $sql = "INSERT INTO CTRLTRCKIDACS (NRCTRLTRCKIDACS, DSTRACKINGID, DSTPDOCUMENTO, DSNMEMPRESAORIG, DTDISPONIVEL, DTULTATU, DTINCLUSAO, NRORG, NRORGINCLUSAO, NRORGULTATU, IDATIVO) 
                        VALUES (:NRCTRLTRCKIDACS, :DSTRACKINGID, :DSTPDOCUMENTO, :DSNMEMPRESAORIG, TO_DATE(:DTDISPONIVEL, 'DD/MM/YYYY HH24:MI:SS'), SYSDATE, SYSDATE, :NRORG, :NRORG, :NRORG, :IDATIVO) ";
                $statement = oci_parse(self::$conn, $sql);
                
                oci_bind_by_name($statement, ':NRCTRLTRCKIDACS', $NRCTRLTRCKIDACS);
                oci_bind_by_name($statement, ':DSTRACKINGID', $msgDisponivel->trkIdIn);
                oci_bind_by_name($statement, ':DSTPDOCUMENTO', $msgDisponivel->tipoDocumento);
                oci_bind_by_name($statement, ':DSNMEMPRESAORIG', $msgDisponivel->nmeEmpresaOrigem);
                oci_bind_by_name($statement, ':DTDISPONIVEL', $msgDisponivel->dtaHoraDisp);
                oci_bind_by_name($statement, ':NRORG', self::$nrorg);
                oci_bind_by_name($statement, ':IDATIVO', $IDATIVO);
                
                oci_execute($statement, OCI_NO_AUTO_COMMIT);
            }
        }
    }
    
    //Verifica se o documento já existe no banco de dados
    private static function verifyTrackingId($msgDisponivel){
        $trackingId = $msgDisponivel->trkIdIn;
        $sql = "SELECT COUNT(*) QT FROM CTRLTRCKIDACS WHERE DSTRACKINGID = :DSTRACKINGID";
        $statement = oci_parse(self::$conn, $sql);
        oci_bind_by_name($statement, ':DSTRACKINGID', $trackingId);
        oci_execute($statement, OCI_NO_AUTO_COMMIT);
        $countIdTrack = oci_fetch_assoc($statement)['QT'];
        if($countIdTrack > 0){
            return false;
        }else{
            return true;
        }
    }
    
    private static function verifyContTrackingId($msgDisponivel){
        $sql = "SELECT COUNT(*) QT FROM CONTTRCKIDACS 
                    WHERE NRCTRLTRCKIDACS = :NRCTRLTRCKIDACS
                    AND DSTRACKINGID      = :DSTRACKINGID
                    AND NRORG             = :NRORG";
        $statement = oci_parse(self::$conn, $sql);
        oci_bind_by_name($statement, ':NRCTRLTRCKIDACS', $msgDisponivel['NRCTRLTRCKIDACS'] );
        oci_bind_by_name($statement, ':DSTRACKINGID', $msgDisponivel['DSTRACKINGID'] );
        //oci_bind_by_name($statement, ':DSCONTTRCKIDACS', $msgDisponivel['DSCONTTRCKIDACS'] );
        oci_bind_by_name($statement, ':NRORG', $msgDisponivel['NRORG'] );
        oci_execute($statement, OCI_NO_AUTO_COMMIT);
        $countIdTrack = oci_fetch_assoc($statement)['QT'];
        if($countIdTrack > 0){
            return false;
        }else{
            return true;
        }
    }

    private static function controlaConteudoMsgPorTrackId($msgsDisponiveis){
        foreach ($msgsDisponiveis as $mensagemDisponivel) {
            
            $sql = "SELECT NRCTRLTRCKIDACS FROM CTRLTRCKIDACS WHERE DSTRACKINGID = :DSTRACKINGID";
            $statement = oci_parse(self::$conn, $sql);
            oci_bind_by_name($statement, ':DSTRACKINGID', $mensagemDisponivel->trkIdIn);
            oci_execute($statement, OCI_NO_AUTO_COMMIT);
            $NRCTRLTRCKIDACS = oci_fetch_assoc($statement)['NRCTRLTRCKIDACS'];
            
            $infoDocumento = new \SoapClient(RECUPERAMSG, array("keep_alive" => true));
            if(!$infoDocumento){
                throw new \Exception("Não foi possível obter conexão para recuperar arquivos.");
            }
                
            $infoDocumento->__setSoapHeaders(self::$header);
            
            $trackingId = [ 'trackingId' => $mensagemDisponivel->trkIdIn ];
            $response = $infoDocumento->__soapCall('process', array($trackingId));
            
            /*
            * statusRequest : Utilizado para verificar se o documento foi encontrado com sucesso através do trackingId.
            * mensagemCodificada : Utilizado para verificar se o conteúdo do documento está codificado na base64.
            */
            
            $statusRequest = $response->dscStatusMensagem;
    		/*if($statusRequest == "Erro"){
    			throw new \Exception("Não foi possível encontrar dados para o tracking id: ".$trackingId["trackingId"]);
    		}*/
    		$conteudoMensagem = $response->dscConteudoMensagem;
    		
    		$mensagemCodificada = $response->flgCompactacao;
            if($mensagemCodificada){
                $conteudoMensagem = AccesstageWS::decodificaBase64($conteudoMensagem);
            }
            
            $data = array();
            $data['NRCTRLTRCKIDACS'] = $NRCTRLTRCKIDACS;
            $data['DSTRACKINGID'] = $mensagemDisponivel->trkIdIn;
            //$data['DSCONTTRCKIDACS'] = $conteudoMensagem;
            $data['NRORG'] = self::$nrorg;
                
    		if(AccesstageWS::verifyContTrackingId($data)){
             
                $IDATIVO = 'S';
                
                $sql = "INSERT INTO CONTTRCKIDACS (NRCTRLTRCKIDACS, DSTRACKINGID, DSCONTTRCKIDACS, DTULTATU, DTINCLUSAO, NRORG, NRORGINCLUSAO, NRORGULTATU, IDATIVO) 
                            VALUES (:NRCTRLTRCKIDACS, :DSTRACKINGID, :DSCONTTRCKIDACS, SYSDATE, SYSDATE, :NRORG, :NRORG, :NRORG, :IDATIVO) ";
                $statement = oci_parse(self::$conn, $sql);
                    
                oci_bind_by_name($statement, ':NRCTRLTRCKIDACS', $NRCTRLTRCKIDACS);
                oci_bind_by_name($statement, ':DSTRACKINGID', $mensagemDisponivel->trkIdIn);
                oci_bind_by_name($statement, ':DSCONTTRCKIDACS', $conteudoMensagem);
                oci_bind_by_name($statement, ':NRORG', self::$nrorg);
                oci_bind_by_name($statement, ':IDATIVO', $IDATIVO);
                
                oci_execute($statement, OCI_NO_AUTO_COMMIT);
    		}
            
        }
    }
    
    //Para mais informações consulte a documentação sobre a função base64_decode: http://php.net/manual/pt_BR/function.base64-decode.php
    //Função utilizada para decodificar mensagens codificadas na base64. 
    private static function decodificaBase64($msgCodificada){
        return base64_decode($msgCodificada);
    }
    
    /* function newCode($counterName, $size, $nrSeq, $qtd)
     * Gera um NRSEQUENCIAL para a tabela especifícada para ser usado em uma operação de insert.
     *
     * @param counterName: nome da tabela
     * @param size: tamanho do inteiro gerado. O default é 20
     * @param nrSeq: NRSEQUENCIAL inicial. O default é não passar nenhum paramêtro para que se gere o NRSEQUENCIAL imediatamente
     * anterior ao mais recente na tabela.
     * @param qtd: quantidade de códigos que serão gerados. O default é sempre 1
     *
     * @return o NRSEQUENCIAL gerado pela procedure que é armazenado na variável nrSeq
     */
    private static function newCode($counterName, $size = 20, $nrSeq = '', $qtd = 1){
        $sql = 'CALL NOVO_CODIGO_NT(:P_CONTADOR, :P_SEQUENCIAL, :P_QTDE)';
        $statement = oci_parse(self::$conn, $sql);
        oci_bind_by_name($statement, ':P_CONTADOR', $counterName);
        oci_bind_by_name($statement, ':P_SEQUENCIAL', $nrSeq, $size);
        oci_bind_by_name($statement, ':P_QTDE', $qtd);
        oci_execute($statement, OCI_NO_AUTO_COMMIT);
        
        if (!$nrSeq)
            throw new \Exception("Não foi possível gerar chave primária para a tabela -> ".$counterName);
            
        return $nrSeq;
    }
    
   /* private static function confirmaRetiradaArquivo($arquivo){
        $arquivo['NMARQUIVORET'] = 'TESTE';
        
        //Formata data para o padrão AccesStage
        $DTRETIRADA = date("Y-m-dTG:i:s.000P");
        $DTRETIRADA = str_replace('BRT','T', $DTRETIRADA);
        
        $arquivo['DTRETIRADA'] = $DTRETIRADA;
        
        $arquivo["DSTRACKINGID"] = $arquivo["DSTRACKINGID"];
        
        $msgConfirmada = new \SoapClient(CONFIRMRETIRADA, array("keep_alive" => true));
        if(!$msgConfirmada){
            throw new \Exception("Não foi possível obter conexão para confirmar a retirada de arquivos.");
        }
        $msgConfirmada->__setSoapHeaders(self::$header);
        
        $parametro = ['trackingID' => $arquivo["DSTRACKINGID"], 'dataRetirada' => $arquivo['DTRETIRADA'], 'nmeArquivo' => $arquivo['NMARQUIVORET']];
        
        $response = $msgConfirmada->__soapCall('process', array($parametro));
        if($response->dscStatusRetirada != "Ok"){
            throw new \Exception("Não foi possível retirar o arquivo com o trakingId: ".$arquivo["DSTRACKINGID"]);
        }

        AccesStageWS::updateControleAcessWS($arquivo);

    }*/
    
    /*private static function updateControleAcessWS($arquivo){
        $sql = "UPDATE CTRLTRCKIDACS
                SET NMARQUIVORET = :NMARQUIVORET,
                    DTRETIRADA   = :DTRETIRADA
                WHERE DSTRACKINGID    = :DSTRACKINGID
                  AND NRCTRLTRCKIDACS = :NRCTRLTRCKIDACS";
        
        $statement = oci_parse(self::$conn, $sql);
        oci_bind_by_name($statement, ':NMARQUIVORET', $arquivo["NMARQUIVORET"]);
        oci_bind_by_name($statement, ':DTRETIRADA'  , $DTRETIRADA);
        oci_bind_by_name($statement, ':NMARQUIVORET', $arquivo["DSTRACKINGID"]);
        oci_bind_by_name($statement, ':NMARQUIVORET', $arquivo["NRCTRLTRCKIDACS"]);
        oci_execute($statement, OCI_NO_AUTO_COMMIT);

    }*/
    
    public function importaExtratoOFX() {
        /*
            Pega todos os dados do AccesStage que não foram concluidos e procuram o conteudo que seja
            de um arquivo OFX
        */
        $sql = "SELECT * FROM CTRLTRCKIDACS WHERE IDSTATIMPORT <> 'C'";
        $statement = oci_parse(self::$conn, $sql);
        //oci_bind_by_name($statement, ':DSTRACKINGID', $mensagemDisponivel->trkIdIn);
        oci_execute($statement, OCI_NO_AUTO_COMMIT);
        oci_fetch_all($statement, $msgsDisponiveis, null, null, OCI_FETCHSTATEMENT_BY_ROW);
        
        foreach ($msgsDisponiveis as $mensagemDisponivel) {
            $sql = "SELECT DSCONTTRCKIDACS FROM CONTTRCKIDACS WHERE DSTRACKINGID = :DSTRACKINGID";
            //$sql = "SELECT DSCONTTRCKIDACS FROM CONTTRCKIDACS WHERE DSTRACKINGID = 201907020411386";
            $statement = oci_parse(self::$conn, $sql);
            oci_bind_by_name($statement, ':DSTRACKINGID', $mensagemDisponivel['DSTRACKINGID']);
            //oci_bind_by_name($statement, ':DSTRACKINGID', 201907020411386);
            oci_execute($statement, OCI_NO_AUTO_COMMIT);
            $data = oci_fetch_row($statement);
            if($data){
                $DSCONTTRCKIDACS = $data[0]->load();
                
                if ((strpos($DSCONTTRCKIDACS,'OFXHEADER')) !== false){
                    self::readFile($DSCONTTRCKIDACS, $mensagemDisponivel['DSTRACKINGID']);
                }
            }
        }
    }
    
    
    
    /**************************** Importação de Extrato Bancário ************************/

    /**
     * Essa função tem como finalidade verificar se o arquivo importado é do tipo TXT ou
     * OFX e chamar a função comrrespindende para lelos. Ambas as funções iram retornar
     * um array de duas posições, onde a primeira se trata dos dados do cabeçalho da
     * widget de imporatação, e a segunda posição as movimentações que deverão ser
     * salvas, e que iram aparecer no grid.
     *
     * @param  DTO\Request\Row  $request  row, ou conjunto do rows (dataset) que sera passado do front
     * @param  DTO\Response     $response o que sera retornado para o backend
     */
    public function readFile($conteudo, $dstrackingid) {
        
        /*
            Função de leitura do OFX do extrato bancario espera um arquivo codificado.
        */
        $conteudo = base64_encode($conteudo);
        $dataSets = $this->Bco54200Service->readFileOfx($conteudo, self::$nrorg, true);
        
        if(isset($dataSets['Error'])){
            var_dump($dataSets['Error']);
        } else {
            
            $rows = $dataSets[1];
            var_dump($dstrackingid);
            $rows = $this->Bco54200Service->removeMovimentosExistentesNoSistema($rows);
            $rowsArray = $this->Bco54200Service->removeMovimentosDuplicadosNoArquivo($rows);
            $rows = $rowsArray[0];
            $movimentosDuplicados = $rowsArray[1];
            if (count($rows)){
                foreach($rows as $row) {
                    
                    //Faz as validações referentes a tabela de extrato, EXTRBANCO
                    $row = $this->Bco54200Service->validaExtrato($row);
                    $this->entityManager->flush();

                    //Quando novo, insere a movimentação na tabela MOVREAL, quando edição, apenas
                    //updata o registro
                    if($row['__is_new']){
                        $this->Bco54200Service->insertMovReal($row);
                    } else {
                        $this->Bco54200Service->updateMovReal($row);
                    }
                    $this->entityManager->flush();

                    //Updata o extrato, recalculando o seu valor final.
                    $this->Bco54200Service->recalculoExtrato($row);
                    $this->entityManager->flush();
                    
                    $sql = "UPDATE CTRLTRCKIDACS SET IDSTATIMPORT = 'C'
                        WHERE IDSTATIMPORT <> 'C'
                        AND DSTRACKINGID = :DSTRACKINGID";
                    $statement = oci_parse(self::$conn, $sql);
                    oci_bind_by_name($statement, ':DSTRACKINGID',  $dstrackingid);
                    oci_execute($statement, OCI_NO_AUTO_COMMIT);
                    
        		    //A geração de log foi cancelada ou existe deletecascae que impede o log
                    //$this->Bco54200Service->geraLog($row);
                    //$this->entityManager->flush();
                }
            }else{
                var_dump("Os dados do arquivo ja foram importados no sistema.");
            }
            
            //$connection->commit();
            //$response->addDataSet(new DataSet("MOVDUPLICADOS", $movimentosDuplicados));
            //$dataSetExtr = new DataSet('extrbanco', $dataSets[0]);
            //$dataSetMov = new DataSet('movreal', $dataSets[1]);
            //$response->setDataSets(array($dataSetExtr, $dataSetMov));
        
        }
    }
   
    public function importaRetornoCNAB() {
        
        /*
            Função de leitura do OFX do extrato bancario espera um arquivo codificado.
        */
        $sql = "SELECT * FROM CTRLTRCKIDACS WHERE IDSTATIMPORT <> 'C'";
        $statement = oci_parse(self::$conn, $sql);
        //oci_bind_by_name($statement, ':DSTRACKINGID', $mensagemDisponivel->trkIdIn);
        oci_execute($statement, OCI_NO_AUTO_COMMIT);
        oci_fetch_all($statement, $msgsDisponiveis, null, null, OCI_FETCHSTATEMENT_BY_ROW);
        
        foreach ($msgsDisponiveis as $mensagemDisponivel) {
            /* 
            partindo do pressuposto que as informações a respeito do CNAB estarao no 
            */
            if ((strpos($mensagemDisponivel['DSTPDOCUMENTO'],'CNAB')) !== false){
                
                $sql = "SELECT DSCONTTRCKIDACS FROM CONTTRCKIDACS WHERE DSTRACKINGID = :DSTRACKINGID";
                //$sql = "SELECT DSCONTTRCKIDACS FROM CONTTRCKIDACS WHERE DSTRACKINGID = 201907020411386";
                $statement = oci_parse(self::$conn, $sql);
                oci_bind_by_name($statement, ':DSTRACKINGID', $mensagemDisponivel['DSTRACKINGID']);
                //oci_bind_by_name($statement, ':DSTRACKINGID', 201907020411386);
                oci_execute($statement, OCI_NO_AUTO_COMMIT);
                $data = oci_fetch_row($statement);
                if($data){
                    $DSCONTTRCKIDACS = $data[0]->load();
                    
                    if ($DSCONTTRCKIDACS){
                        var_dump('$DSCONTTRCKIDACS: '.$mensagemDisponivel['NRCTRLTRCKIDACS']);
                        self::readFileCnab($DSCONTTRCKIDACS, $mensagemDisponivel['DSTRACKINGID'], $mensagemDisponivel['DSNMEMPRESAORIG']);
                    }
                }
            }
        }
    }
    
    public function readFileCnab($conteudo, $dstrackingid, $dsnmempresaorig) {
        
        $row['CDBANCCNAB'] = substr($conteudo, 0, 3) ;
        $row['CDBANCO'] = substr($conteudo, 0, 3) ;
        $row['CDEMPRESA'] = $dsnmempresaorig;
        $row['NRORG'] = self::$nrorg;
        
        $conteudo = base64_encode($conteudo);
        
        $row['CNABRETORNO'] = $conteudo;
        
        //var_dump($row);
        
        $dataSet = $this->RetornoServices->carregaRetorno($row, true);
        
        //$dataSets = $this->Bco54200Service->readFileOfx($conteudo, self::$nrorg, true);
        
        //$dsarqcnabpag = $rows[0]['DSARQCNABPAG'];
        
        $boletos = $dataSet[0];
        $ocorrencias = $dataSet[1];
        
        $this->RetornoServices->insereRetorno($boletos, $ocorrencias, '', $row['CDEMPRESA'], $row['CDBANCO'], self::$nrorg);
        
        //$rows[0]['NOMEARQUIVO']
        
    }
    
    //Formata data recebida pelo webservice (padrão AAAA-MM-DDTHH:MIN:SS.000-02:00) 
    //para o padrão utilizado pelo banco de dados (padrão DD/MM/AAAA HH:MIN:SS)
    private static function formatDate($date){
        return date('d/m/Y H:i:s', strtotime($date));
    }

}

AccesstageWS::executeWs();