<?php

namespace Teknisa\FIN\Modules\ContaPagar\Factory;

use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Teknisa\Model\Geral\Entities\Paramgeral;
use Teknisa\Model\Financeiro\Entities\Ajbolpag;
use Teknisa\Model\Financeiro\Entities\Ajctpagar;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Tipoctpagar;
use Teknisa\Model\Financeiro\Entities\Tipoajpagar;

/**
 * Class AssociacaoTituloFactory
 * @package Teknisa\FIN\Modules\ContaPagar\Factory
 */
class AssociacaoTituloFactory {

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /**
     * Busca Boleto a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @return BoletPagar
     */
    public function buscaBoletpagar($nrorg, $cdfilial, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Boletpagar $boletpagar */
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findOneBy($params);

        return $boletpagar;
    }

    /**
     * Busca Contas a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @return Contapagar
     */
    public function buscaContaspagar($nrorg, $cdfilial, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilibpag']  = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Contapagar $contapagar */
        $contapagar = $this->entityManager->getRepository(Contapagar::class)->findBy($params);

        return $contapagar;
    }

    /**
     * Busca Tipo Conta Pagar
     * @param $nrorg
     * @param $cdtpctpagar
     * @return Tipoctpagar
     */
    public function buscaTipoctpagar($nrorg, $cdtpctpagar){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdtpctpagar'] = $cdtpctpagar;

        /** @var Tipoctpagar $tipoctpagar */
        $tipoctpagar = $this->entityManager->getRepository(Tipoctpagar::class)->findOneBy($params);

        return $tipoctpagar;
    }

    /**
     * Busca Ajustes Conta Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @return Ajctpagar
     */
    public function buscaAjsctpagar($nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrseqvenpag']  = $nrseqvenpag;
        $params['nrparcelapag'] = $nrparcelapag;

        /** @var Ajctpagar $ajctpagar */
        $ajctpagar = $this->entityManager->getRepository(Ajctpagar::class)->findBy($params);

        return $ajctpagar;
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdajuctpag
     * @return Ajbolpag
     */
    public function buscaAjbolpag($nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;
        $params['cdajuctpag']  = $cdajuctpag;

        /** @var Ajbolpag $ajbolpag */
        $ajbolpag = $this->entityManager->getRepository(Ajbolpag::class)->findOneBy($params);

        return $ajbolpag;
    }

    /**
     * Busca Ajustes Boleto a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @return Ajbolpag
     */
    public function buscaAjsbolpag($nrorg, $cdfilial, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Ajbolpag $ajbolpag */
        $ajbolpag = $this->entityManager->getRepository(Ajbolpag::class)->findBy($params);

        return $ajbolpag;
    }

    /**
     * @param $nrorg
     * @param $cdajuctpag
     * @return Tipoajpagar
     */
    public function buscaTipoajpagar($nrorg, $cdajuctpag){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdajuctpag'] = $cdajuctpag;

        /** @var Tipoajpagar $tipoajpagar */
        $tipoajpagar = $this->entityManager->getRepository(Tipoajpagar::class)->findOneBy($params);

        return $tipoajpagar;
    }

    /**
     * Insere Boleto Pagar
     * @param $nrorg
     * @param $novocodigo
     * @param $cdfilial
     * @param $valortotal
     * @param $dtvenbolpag
     * @param $cdreducont
     * @param $cdtpbaixapag
     * @param $cdforneced
     * @param $cdempresa
     * @param $idstatbpag
     * @param $idsitucnabp
     * @param $nrparcbolpag
     * @return Boletpagar
     */
    public function insereBoletpagar(
        $nrorg, $novocodigo, $cdfilial, $valortotal, $vrfinalbpag, $dtvenbolpag, $cdreducont, $cdtpbaixapag, $cdforneced,
        $cdempresa, $idstatbpag, $idsitucnabp, $nrparcbolpag, $vrpemulbpag = null, $vrmultabpag = null,
        $vrpemorbpag = null, $vrmordiabpag = null, $cdbancbolp = null, $cdagenbolp = null , $cdccorbolp = null, $cdoperinclusao = null, $idbpagreal = 'N'
    ){
        /** @var Boletpagar $boletpagar */
        $boletpagar = new Boletpagar();

        $boletpagar->setNrorg($nrorg);
        $boletpagar->setNrlanbolpag($novocodigo);
        $boletpagar->setCdfilial($cdfilial);
        $boletpagar->setVrbolpag($valortotal);
        $boletpagar->setVrfinalbpag($vrfinalbpag);
        $boletpagar->setDtvenbolpag($dtvenbolpag);
        $boletpagar->setCdreducont($cdreducont);
        $boletpagar->setCdtpbaixapag($cdtpbaixapag);
        $boletpagar->setCdforneced($cdforneced);
        $boletpagar->setCdempresa($cdempresa);
        $boletpagar->setIdstatbpag($idstatbpag);
        $boletpagar->setIdsitucnabp($idsitucnabp);
        $boletpagar->setNrparcbolpag($nrparcbolpag);
        $boletpagar->setVrpemulbpag($vrpemulbpag);
        $boletpagar->setVrmultabpag($vrmultabpag);
        $boletpagar->setVrpemorbpag($vrpemorbpag);
        $boletpagar->setVrmordiabpag($vrmordiabpag);
        $boletpagar->setCdbancbolp($cdbancbolp);
        $boletpagar->setCdagenbolp($cdagenbolp);
        $boletpagar->setCdccorbolp($cdccorbolp);
        $boletpagar->setCdoperinclusao($cdoperinclusao);
        $boletpagar->setIdbpagreal($idbpagreal);
        $boletpagar->setDtinclusao(DateUtil::getDataAtual());
        

        return $boletpagar;
    }

    /**
     * Insere Ajuste Boleto Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdajuctpag
     * @param $vrajbolpag
     * @return Ajbolpag
     */
    public function insereAjbolpag(
        $nrorg, $cdfilial, $nrlanbolpag, $cdajuctpag, $vrajbolpag
    ){
        /** @var Ajbolpag $ajbolpag */
        $ajbolpag = new Ajbolpag();

        $ajbolpag->setNrorg($nrorg);
        $ajbolpag->setCdfilial($cdfilial);
        $ajbolpag->setNrlanbolpag($nrlanbolpag);
        $ajbolpag->setCdajuctpag($cdajuctpag);
        $ajbolpag->setVrajbolpag($vrajbolpag);
        $ajbolpag->setDtinclusao(DateUtil::getDataAtual());

        return $ajbolpag;
    }

    /**
     * Atualiza Conta Pagar
     * @param Contapagar $contapagar
     * @param $nrlanbolpag
     * @param $cdfilibpag
     * @param $dtatuavenpag
     * @param $dtmovfluxo
     * @param $cdmotipror
     * @return Contapagar
     */
    public function atualizaContapagar(
        Contapagar $contapagar, $nrlanbolpag, $cdfilibpag, $dtatuavenpag, $dtmovfluxo, $cdmotipror
    ){
        $contapagar->setNrlanbolpag($nrlanbolpag);
        $contapagar->setCdfilibpag($cdfilibpag);
        $contapagar->setDtatuavenpag($dtatuavenpag);
        $contapagar->setDtmovfluxo($dtmovfluxo);
        $contapagar->setCdmotipror($cdmotipror);

        return $contapagar;
    }

}