<?php

namespace Teknisa\FIN\Modules\ContaReceber\Factory;

use Teknisa\Model\Financeiro\Entities\Boletreceber;
use Teknisa\Model\Financeiro\Entities\Contareceber;
use Teknisa\Model\Financeiro\Entities\Histnegociacao;
use Financeiro\Entities\Ratctreceber;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;


/**
 * Class ParcelamentoBoletoFactory
 * @package Teknisa\FIN\Modules\ContaReceber\Factory
 */
class ParcelamentoBoletoFactory {

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @return Notafiscal
     */
    public function buscaNotafiscal($nrorg, $cdfilial, $nrlanctonf){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfilial']   = $cdfilial;
        $params['nrlanctonf'] = $nrlanctonf;

        /** @var Notafiscal $notafiscal */
        $notafiscal = $this->entityManager->getRepository(Notafiscal::class)->findOneBy($params);

        return $notafiscal;
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @return Ratctreceber
     */
    public function buscaRatCtReceber($nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrseqvenrec']  = $nrseqvenrec;
        $params['nrparcelarec'] = $nrparcelarec;

        /** @var Ratctreceber $ratctreceber */
        $ratctreceber = $this->entityManager->getRepository(Ratctreceber::class)->findBy($params);

        return $ratctreceber;
    }
    
    /**
     * Busca Boleto a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return Boletreceber
     */
    public function buscaBoletoReceber($nrorg, $cdfilial, $nrlanbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolrec'] = $nrlanbolrec;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findOneBy($params);

        return $boletreceber;
    }
    
    /**
     * Busca Historico de Negociação
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @return Histnegociacao
     */
    public function buscaHistnegociacao($nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrseqvenrec']  = $nrseqvenrec;
        $params['nrparcelarec'] = $nrparcelarec;

        /** @var Histnegociacao $histnegociacao */
        $histnegociacao = $this->entityManager->getRepository(Histnegociacao::class)->findBy($params);

        return $histnegociacao;
    }
    
    /**
     * Busca Boletos Associados ao Boleto de Origem
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return Boletreceber
     */
    public function buscaBoletosOrigem($nrorg, $cdfilial, $nrorigbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrorigbolrec'] = $nrorigbolrec;

        /** @var Boletreceber $boletreceber */
        $boletreceber = $this->entityManager->getRepository(Boletreceber::class)->findBy($params);

        return $boletreceber;
    }
    
    /**
     * Busca Conta Receber
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @return Contareceber
     */
    public function buscaContareceberBoleto($nrorg, $cdfilibolrec, $nrlanbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilibolrec'] = $cdfilibolrec;
        $params['nrlanbolrec' ] = $nrlanbolrec;

        /** @var Contareceber $contareceber */
        $contareceber = $this->entityManager->getRepository(Contareceber::class)->findOneBy($params);

        return $contareceber;
    }
    
    /**
     * Busca Contas Receber
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @return Contareceber
     */
    public function buscaContasreceberBoleto($nrorg, $cdfilibolrec, $nrlanbolrec){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilibolrec'] = $cdfilibolrec;
        $params['nrlanbolrec' ] = $nrlanbolrec;

        /** @var Contareceber $contareceber */
        $contareceber = $this->entityManager->getRepository(Contareceber::class)->findBy($params);

        return $contareceber;
    }
    
    /**
     * Busca Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @return Contareceber
     */
    public function buscaContareceber($nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec){
        /** @var $params */
        $params = array();
        $params['nrorg']         = $nrorg;
        $params['cdfilial']      = $cdfilial;
        $params['nrlanctonf' ]   = $nrlanctonf;
        $params['nrseqvenrec' ]  = $nrseqvenrec;
        $params['nrparcelarec' ] = $nrparcelarec;
        
        /** @var Contareceber $contareceber */
        $contareceber = $this->entityManager->getRepository(Contareceber::class)->findOneBy($params);

        return $contareceber;
    }

    /**
     * Insere Rateio Tipo Conta Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdcentcust
     * @param $cdfiratctrec
     * @param $vrratccctrec
     * @return Ratctreceber
     */
    public static function insereRatctreceber(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust, $cdfiratctrec, $vrratccctrec
    ){
        /** @var Ratctreceber $rattpctr */
        $ratctreceber = new Ratctreceber();
        $ratctreceber->setNrorg($nrorg);
        $ratctreceber->setCdfilial($cdfilial);
        $ratctreceber->setNrlanctonf($nrlanctonf);
        $ratctreceber->setNrseqvenrec($nrseqvenrec);
        $ratctreceber->setNrparcelarec($nrparcelarec);
        $ratctreceber->setCdcentcust($cdcentcust);
        $ratctreceber->setCdfiratctrec($cdfiratctrec);
        $ratctreceber->setVrratccctrec($vrratccctrec);
        $ratctreceber->setDtinclusao(DateUtil::getDataAtual());
        return $ratctreceber;
    }

    /**
     * @param Boletreceber $boletooriginal
     * @param $cdmotiparc
     * @param $idstatbrec
     * @param $nrorigbolrec
     * @param $idimprbolbol
     * @param $idformcadbrec
     * @param $nrparcbolrec
     * @param $dtvenbolrec
     * @param $idsitucnabr
     * @param $vrfinalbrec
     * @param $vrbolrec
     * @return Boletreceber
     */
    public function updataBoletoOriginal(
        Boletreceber $boletooriginal, $cdmotiparc, $idstatbrec, $nrorigbolrec, $idimprbolbol, $idformcadbrec, $nrparcbolrec, $dtvenbolrec, $idsitucnabr, $vrfinalbrec, $vrbolrec
    ){
        $boletooriginal->setVrfinalbrec($vrfinalbrec);
        $boletooriginal->setVrbolrec($vrbolrec);
        $boletooriginal->setCdmotiparc($cdmotiparc);
        $boletooriginal->setNrparcbolrec($nrparcbolrec);
        $boletooriginal->setNrorigbolrec($nrorigbolrec);
        $boletooriginal->setDtvenbolrec($dtvenbolrec);
        $boletooriginal->setIdsitucnabr($idsitucnabr);
        $boletooriginal->setIdstatbrec($idstatbrec);
        $boletooriginal->setIdimprbolbol($idimprbolbol);
        $boletooriginal->setIdformcadbrec($idformcadbrec);
        $boletooriginal->setNrcnabrecrem(null);
        $boletooriginal->setNrbaixarec(null);
        $boletooriginal->setCdfilibairec(null);
        $boletooriginal->setDslinhabarrec(null);
        $boletooriginal->setCdbarrarec(null);
        $boletooriginal->setCdnosnumrec(null);
        $boletooriginal->setTxmensbolbol(null);

        return $boletooriginal;
    }

    /**
     * Busca Conta Original Parcelamento
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return array
     */
    public function buscaContaOriginalParcelamento(
        $nrorg, $cdfilial, $nrlanbolrec
    ){

        /** @var $params */
        $params = array();
        $params['P_NRORG']        = $nrorg;
        $params['P_CDFILIAL' ]    = $cdfilial;
        $params['P_NRLANBOLREC']  = $nrlanbolrec;

        $sql = "SELECT
                  CR.CDFILIAL,
                  CR.NRLANCTONF,
                  NF.NRNOTAFISC,
                  NF.CDSERIE,
                  CR.NRSEQVENREC,
                  CR.NRPARCELAREC,
                  CR.NRPARCELARECVISU,
                  CR.DSCONTAREC,
                  CR.DTORIGVENREC,
                  CR.VRORIGREC,
                  CR.DTATUAVENREC,
                  CR.VRATUAREC,
                  CR.NRLANBOLREC,
                  CR.CDCLASSFINA,
                  CR.CDEMPRESA,
                  CR.CDFIMOVFLREC,
                  CR.DTMOVFLUXO,
                  CR.IDMOVFLUXO,
                  CR.CDMOTIPARC,
                  CR.QTDIASVENREC,
                  CR.IDSTATREC,
                  CR.CDFILIBOLREC,
                  CR.CDTPCTRECEB,
                  CR.CDFILIALDUP,
                  CR.NRLANCTODUP,
                  AJ.VRAJUSTE,
                  CR.NRORG
                FROM
                  NOTAFISCAL   NF,
                  CONTARECEBER CR,
                  (SELECT A.CDFILIAL, A.NRLANCTONF, A.NRSEQVENREC, A.NRPARCELAREC, A.NRORG,
                          SUM(DECODE(TA.IDAJUCTREC, 'A', A.VRAJUSTEREC, A.VRAJUSTEREC * (-1))) AS VRAJUSTE
                     FROM AJCTRECEBER A, TIPOAJRECEBER TA
                    WHERE A.NRORG      = TA.NRORG
                    AND   A.CDAJUCTREC = TA.CDAJUCTREC
                    GROUP BY A.CDFILIAL, A.NRLANCTONF, A.NRSEQVENREC, A.NRPARCELAREC, A.NRORG) AJ
                WHERE
                  (CR.CDFILIBOLREC = :P_CDFILIAL       ) AND
                  (CR.NRLANBOLREC  = :P_NRLANBOLREC    ) AND
                  (CR.NRORG        = :P_NRORG          ) AND
                  (NF.CDFILIAL     = CR.CDFILIAL       ) AND
                  (NF.NRORG        = CR.NRORG          ) AND
                  (NF.NRLANCTONF   = CR.NRLANCTONF     ) AND
                  (CR.CDFILIAL     = AJ.CDFILIAL(+)    ) AND
                  (CR.NRLANCTONF   = AJ.NRLANCTONF(+)  ) AND
                  (CR.NRSEQVENREC  = AJ.NRSEQVENREC(+) ) AND
                  (CR.NRPARCELAREC = AJ.NRPARCELAREC(+))
                ORDER BY
                  CR.CDFILIAL,
                  CR.NRLANCTONF,
                  CR.NRSEQVENREC,
                  CR.NRPARCELAREC";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Busca Ajuste Conta
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @return array
     */
    public function buscaAjusteConta(
        $nrorg, $cdfilibolrec, $nrlanbolrec
    ){

        /** @var $params */
        $params = array();
        $params['P_NRORG']        = $nrorg;
        $params['P_CDFILIBOLREC'] = $cdfilibolrec;
        $params['P_NRLANBOLREC']  = $nrlanbolrec;

        $sql = "SELECT
                   AR.CDFILIAL,
                   AR.NRLANCTONF,
                   AR.NRSEQVENREC,
                   AR.NRPARCELAREC,
                   AR.DTAJUSTEREC,
                   AR.DSAJUSTEREC,
                   AR.CDAJUCTREC,
                   TA.IDAJUCTREC,
                   AR.CDCLASSFINA,
                   AR.CDEMMFAJCNTR,
                   AR.CDFIMFAJCNTR,
                   AR.DTMOVFLUXO,
                   AR.IDMOVFLUXO,
                   AR.VRAJUSTEREC,
                   AR.NRORG
                FROM
                   AJCTRECEBER   AR,
                   TIPOAJRECEBER TA,
                   CONTARECEBER  CT
                WHERE
                   (CT.CDFILIBOLREC = :P_CDFILIBOLREC) AND
                   (CT.NRLANBOLREC  = :P_NRLANBOLREC)  AND
                   (AR.CDFILIAL     = CT.CDFILIAL)     AND
                   (AR.NRLANCTONF   = CT.NRLANCTONF)   AND
                   (AR.NRSEQVENREC  = CT.NRSEQVENREC)  AND
                   (AR.NRPARCELAREC = CT.NRPARCELAREC) AND
                   (TA.CDAJUCTREC   = AR.CDAJUCTREC)   AND
                   (CT.NRORG        = :P_NRORG)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Busca Max Conta Parcelada Receber
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @return array
     */
    public function buscaMaxCntParcReceber(
        $nrorg, $cdfilibolrec, $nrlanctonf, $nrseqvenrec
    ){

        /** @var $params */
        $params = array();
        $params['P_NRORG']       = $nrorg;
        $params['P_CDFILIAL']    = $cdfilibolrec;
        $params['P_NRLANCTONF']  = $nrlanctonf;
        $params['P_NRSEQVENREC'] = $nrseqvenrec;

        $sql = "SELECT
                   MAX(NRPARCELAREC) AS MAXNRPARCREC
                FROM
                   CONTARECEBER
                WHERE
                   CDFILIAL    = :P_CDFILIAL    AND
                   NRLANCTONF  = :P_NRLANCTONF  AND
                   NRSEQVENREC = :P_NRSEQVENREC AND
                   NRORG       = :P_NRORG";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Busca Valor Rateio Conta
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @return array
     */
    public function buscaValorRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec
    ){

        /** @var $params */
        $params = array();
        $params['P_NRORG']        = $nrorg;
        $params['P_CDFILIAL' ]    = $cdfilial;
        $params['P_NRLANCTONF']   = $nrlanctonf;
        $params['P_NRSEQVENREC']  = $nrseqvenrec;
        $params['P_NRPARCELAREC'] = $nrparcelarec;

        $sql = "SELECT
                   RATCTRECEBER.CDFILIAL,
                   RATCTRECEBER.NRLANCTONF,
                   RATCTRECEBER.NRSEQVENREC,
                   RATCTRECEBER.NRPARCELAREC,
                   RATCTRECEBER.CDCENTCUST,
                   RATCTRECEBER.CDFIRATCTREC,
                   RATCTRECEBER.VRRATCCCTREC,
                   CONTARECEBER.VRATUAREC,
                   ((RATCTRECEBER.VRRATCCCTREC * 100)/CONTARECEBER.VRATUAREC) AS NRPERCENTUAL
                FROM
                   RATCTRECEBER,
                   CONTARECEBER
                WHERE
                   RATCTRECEBER.CDFILIAL     = :P_CDFILIAL               AND
                   RATCTRECEBER.NRLANCTONF   = :P_NRLANCTONF             AND
                   RATCTRECEBER.NRSEQVENREC  = :P_NRSEQVENREC            AND
                   RATCTRECEBER.NRPARCELAREC = :P_NRPARCELAREC           AND
                   RATCTRECEBER.NRORG        = :P_NRORG                  AND
                   RATCTRECEBER.CDFILIAL     = CONTARECEBER.CDFILIAL     AND
                   RATCTRECEBER.NRLANCTONF   = CONTARECEBER.NRLANCTONF   AND
                   RATCTRECEBER.NRSEQVENREC  = CONTARECEBER.NRSEQVENREC  AND
                   RATCTRECEBER.NRPARCELAREC = CONTARECEBER.NRPARCELAREC";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

}