<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Ajbolrec;
use Financeiro\Entities\Contareceber;
use Financeiro\Entities\Movfluxo;
use Fiscal\Entities\Notafiscal;
use Teknisa\FIN\Modules\ContaReceber\Factory\AjusteBoletoFactory as DtsAjustesBoletoFactory;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\Model\Financeiro\Entities\Adiantareceb;
use Teknisa\Model\Financeiro\Entities\Devolucaorec;

class AjusteBoletoService {

    protected $fluxoDeCaixaService;
    protected $dtsAjustesBoleto;
    protected $entityManager;

    function __construct(EntityManager $entityManager, DtsAjustesBoletoFactory $dtsAjustesBoleto, FluxoDeCaixaService $fluxoDeCaixaService)
    {
        $this->entityManager = $entityManager;
        $this->dtsAjustesBoleto = $dtsAjustesBoleto;
        $this->fluxoDeCaixaService = $fluxoDeCaixaService;
    }

    public function validaStatusBoleto($nrorg, $cdfilial, $nrlanbolrec){

        try {
            $boleto = $this->dtsAjustesBoleto->getBoleto($nrorg, $cdfilial, $nrlanbolrec);
            if ($boleto->getIdstatbrec() == 'E') {
                throw new \Exception('Operação bloqueada. Título emitido não pode ser alterado.');
            } elseif ($boleto->getIdstatbrec() == 'D') {
                throw new \Exception('Operação bloqueada. Título descontado não pode ser alterado.');
            } elseif ($boleto->getIdstatbrec() == 'B') {
                throw new \Exception('Operação bloqueada. Título baixado não pode ser alterado.');
            } elseif ($boleto->getNrcnabrecrem()) {
                throw new \Exception('Operação bloqueada. Título enviado, via CNAB, não pode ser alterado.');
            }
        } catch (\Exception $e) {
            throw $e;
        }

    }

    public function inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $vrajbolrec, $idorigajbrec = '1'){
        try {
            $ajuste = $this->dtsAjustesBoleto->inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $vrajbolrec, $idorigajbrec);
            $this->entityManager->persist($ajuste);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $vrajuste){
        try {
            if ($idajuctrec == 'D') {
                $vrajuste = -($vrajuste);
            }

            $boleto = $this->dtsAjustesBoleto->atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolrec, $vrajuste);
            if($boleto->getVrfinalbrec() < 0){
                throw new \Exception('Valor do ajuste deve ser menor que valor do boleto.');
            }
            $this->entityManager->persist($boleto);

            $this->dtsAjustesBoleto->atualizaFluxoCaixa($nrorg, $cdfilial, $nrlanbolrec, $vrajuste);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function rotinaInsercaoAjuste($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $cdajuctrec, $vrajbolrec){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->insereAjusteReceber($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $cdajuctrec, $vrajbolrec);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function insereAjusteReceber($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $cdajuctrec, $vrajbolrec, $idorigajbrec = '1'){
        try {
            // Busca Tipo de Ajuste a Receber
            $tipoajreceber = $this->dtsAjustesBoleto->buscaTipoajreceber($nrorg, $cdajuctrec);
            $idajuctrec = $tipoajreceber->getIdajuctrec();

            $this->validaCodigoAjusteExistente($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec);

            $this->validaStatusBoleto($nrorg, $cdfilial, $nrlanbolrec);

            // Busca Ajuste Titulo
            /** @var Ajbolrec $ajbolrec */
            $ajbolrec = $this->dtsAjustesBoleto->getAjuste($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec);

            // Verifica Existência Ajuste Titulo
            if(!$ajbolrec) {
                // Insere Ajuste Titulo
                self::inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $vrajbolrec, $idorigajbrec);
            }else{
                // Atualiza Ajuste Titulo
                self::atualizaAjusteBoleto($ajbolrec, $vrajbolrec);
            }

            $this->atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $vrajbolrec);

            //Contabilizar

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function validaCodigoAjusteExistente($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec){
        try {
            if ($this->dtsAjustesBoleto->getAjuste($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec) instanceof Ajbolrec) {
                throw new \Exception('Operação bloqueada. Registro já cadastrado.');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function rotinaExclusaoAjuste($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $idajuctrec, $vrajbolrec){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->rotinaExclusaoAjusteReceber($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $idajuctrec, $vrajbolrec);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function rotinaExclusaoAjusteReceber($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $idajuctrec, $vrajbolrec){
        try {
            $this->validaStatusBoleto($nrorg, $cdfilial, $nrlanbolrec);

            // Insere Ajuste Boleto
            self::deletaAjusteBoletoReceber($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec);

            $this->dtsAjustesBoleto->deletarAjuste($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec);

            $idajuctrec = $idajuctrec == 'A' ? 'D' : 'A';
            $this->atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $vrajbolrec);

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Deleta Ajuste Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdajuctrec
     * @return Ajbolrec
     * @throws \Exception
     */
    public function deletaAjusteBoletoReceber($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec){
        try {

            /** @var Ajbolrec $ajbolrec */
            $ajbolrec = $this->dtsAjustesBoleto->buscaAjbolrec($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec);

            // Deleta Adiantamento Receber
            self::deletaAdiantamentoReceber(
                $nrorg, $cdfilial, $nrlanbolrec, $ajbolrec->getCdfilial(), $ajbolrec->getCdajuctrec()
            );

            // Deleta Devolução Receber
            self::deletaDevolucaoReceber($nrorg, $cdfilial, $nrlanbolrec, $ajbolrec->getCdajuctrec());

            if ($ajbolrec instanceof Ajbolrec)
                $this->entityManager->remove($ajbolrec);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Codigo Ajuste Existente
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdfiliajbolr
     * @param $cdajuctrec
     * @throws \Exception
     */
    public function deletaAdiantamentoReceber(
        $nrorg, $cdfilial, $nrlanbolrec, $cdfiliajbolr, $cdajuctrec
    ){
        try {

            /** @var Adiantareceb $adiantareceb */
            $adiantareceb = $this->dtsAjustesBoleto->buscaAdiantareceb(
                $nrorg, $cdfilial, $nrlanbolrec, $cdfiliajbolr, $cdajuctrec
            );

            if ($adiantareceb instanceof Adiantareceb)
                $this->entityManager->remove($adiantareceb);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Codigo Ajuste Existente
     * @param $nrorg
     * @param $cdfilibolrec
     * @param $nrlanbolrec
     * @param $cdajuctrec
     * @throws \Exception
     */
    public function deletaDevolucaoReceber(
        $nrorg, $cdfilibolrec, $nrlanbolrec, $cdajuctrec
    ){
        try {

            /** @var Devolucaorec $devolucaorec */
            $devolucaorec = $this->dtsAjustesBoleto->buscaDevolucaorec(
                $nrorg, $cdfilibolrec, $nrlanbolrec, $cdajuctrec
            );

            if ($devolucaorec instanceof Devolucaorec)
                $this->entityManager->remove($devolucaorec);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere Ajuste Adiantamento/Devolução Título a Receber
     * @param $nrorg
     * @throws \Exception
     */
    public function insereAjusteAdiantDevolCliente(
        $nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $cdajuctrec, $vrajbolrec
    ){
        try {

            // Busca Ajuste Titulo
            /** @var Ajbolrec $ajbolrec */
            $ajbolrec = $this->dtsAjustesBoleto->getAjuste($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec);

            // Verifica Existência Ajuste Titulo
            if(!$ajbolrec) {
                // Insere Ajuste Titulo
                self::inserirAjusteBoleto($nrorg, $cdfilial, $nrlanbolrec, $cdajuctrec, $vrajbolrec);
            }else{
                // Atualiza Ajuste Titulo
                self::atualizaAjusteBoleto($ajbolrec, $vrajbolrec);
            }

            // Atualiza Valor Boleto
            self::atualizaValorBoleto($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $vrajbolrec);

            // Atualiza Fluxo de Caixa
            self::atualizaFluxoCaixa($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $vrajbolrec);

            // Recalcular Multa e Mora

            // Altera Tipo de Recebimento

            // Contabiliza na BAIXA

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza Ajuste Boleto
     * @param $ajbolrec
     * @param $vrajbolrec
     * @throws \Exception
     */
    public function atualizaAjusteBoleto(Ajbolrec $ajbolrec, $vrajbolrec){
        try {
            // Atualiza Valor Titulo A Receber
            $vrajbolrec += $ajbolrec->getVrajbolrec();

            // Busca Ajuste Titulo A Receber
            /** @var Ajbolrec $ajbolrec */
            $ajbolrec = $this->dtsAjustesBoleto->atualizaAjbolrec(
                $ajbolrec, $vrajbolrec
            );

            $this->entityManager->persist($ajbolrec);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza Valor Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $idajuctrec
     * @param $vrajbolrec
     * @throws \Exception
     */
    public function atualizaFluxoCaixa($nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $vrajbolrec){
        try {

            /** @var Contareceber $contareceber */
            $contareceber = $this->dtsAjustesBoleto->buscaContareceber($nrorg, $cdfilial, $nrlanbolrec);

            /** @var Notafiscal $notafiscal */
            $notafiscal = $this->dtsAjustesBoleto->buscaNotafiscal($contareceber->getNrorg(), $contareceber->getCdfilial(), $contareceber->getNrlanctonf());

            // Verifica Tipo de Movimentação de Fluxo
            $idmovfluxo = ($notafiscal->getIdstatus() == 'V')?'P':'E';

            // Verifica Tipo de Ajuste[Acrescer|Decrescer]
            $vrajbolrec = ($idajuctrec == 'A')?$vrajbolrec:-($vrajbolrec);

            // Atualiza Fluxo de Caixa
            /** @var Movfluxo $movfluxo */
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $nrorg, $contareceber->getCdempresa(), $contareceber->getCdfimovflrec(), $contareceber->getCdclassfina(), $contareceber->getDtatuavenrec(), $vrajbolrec, $idmovfluxo, 'E'
            );
            $this->entityManager->persist($movfluxo);

        } catch (\Exception $e) {
            throw $e;
        }
    }

}