<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Pix;

use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util\Api;
use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util\ApiItau;
use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Util\Payload;
use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Banco\GeraPixEfiService;
use Teknisa\FIN\Modules\ContaReceber\Services\Pix\Banco\GeraPixItauService;
use Doctrine\ORM\EntityManager;

use Mpdf\QrCode\QrCode;
use Mpdf\QrCode\Output;

class GeraPixService {
    
	const CODIGO_COMPENSACAO_SANTANDER  = "033";
    const CODIGO_COMPESACAO_BRADESCO    = "237";
    const CODIGO_COMPESACAO_ITAU        = "341";
    const CODIGO_COMPESACAO_EFI         = "364";
    const CODIGO_COMPESACAO_BRASIL      = "001";
    const CODIGO_COMPESACAO_CAIXA       = "104";
    const CODIGO_COMPESACAO_SICOOB      = "756";
    const CODIGO_COMPESACAO_SICREDI     = "748";
    const CODIGO_COMPESACAO_SOFISA      = "637";
    const CODIGO_COMPESACAO_VOTORANTIM  = "655";
    const CODIGO_COMPENSACAO_DAYCOVAL   = "707";
    const CODIGO_COMPENSACAO_BANRISUL   = "041";
    
    function __construct(EntityManager $entityManager, GeraPixEfiService $geraPixEfiService, GeraPixItauService $geraPixItauService) {
        $this->entityManager = $entityManager;
        $this->geraPixEfiService = $geraPixEfiService;
        $this->geraPixItauService = $geraPixItauService;
    }
    
    /**
     * Cria Cobrança Vencimento Pix
     */
    public function criaCobrancaVPix($cdbanco, $cdclientid, $cdclientsecret, $dsdircertificado, $cdsenhacertif, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao, $homologacao = true){
        try {
            
            if($cdbanco == self::CODIGO_COMPESACAO_ITAU){
                $payloadQrCode = $this->geraPixItauService->criaCobrancaVPix($cdclientid, $cdclientsecret, $dsdircertificado, $cdsenhacertif, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao, $homologacao);
            }
            elseif($cdbanco == self::CODIGO_COMPESACAO_EFI){
                $payloadQrCode = $this->geraPixEfiService->criaCobrancaVPix($cdclientid, $cdclientsecret, $dsdircertificado, $cdsenhacertif, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao, $homologacao);
            }else $payloadQrCode = null;
            
            return $payloadQrCode;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Altera Cobrança Vencimento Pix
     */
    public function alteraCobrancaVPix($cdbanco, $cdclientid, $cdclientsecret, $dsdircertificado, $cdsenhacertif, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao, $homologacao = true){
        try {
            
            if($cdbanco == self::CODIGO_COMPESACAO_ITAU){
                $payloadQrCode = $this->geraPixItauService->alteraCobrancaVPix($cdclientid, $cdclientsecret, $dsdircertificado, $cdsenhacertif, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao, $homologacao);
            }
            elseif($cdbanco == self::CODIGO_COMPESACAO_EFI){
                $payloadQrCode = $this->geraPixEfiService->alteraCobrancaVPix($cdclientid, $cdclientsecret, $dsdircertificado, $cdsenhacertif, $chavepix, $txtid, $vencimento, $validade, $cliente, $inscricao, $valor, $descricao);
            }else $payloadQrCode = null;
            
            return $payloadQrCode;
        } catch (\Exception $e) {
            throw $e;
        }
    }
}