<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Banco;

use Financeiro\Entities\Cnabrec;
use Geral\Entities\Cliente;
use Geral\Entities\Empresa;
use Financeiro\Entities\Boletreceber;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\LeitorRetorno;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Retorno;

class LeitorRetornoDaycoval extends LeitorRetorno{
    const CODIGO_COMPENSACAO_DAYCOVAL = "707";
    const ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    const ID_TP_TRANSACAO_1 = '1';
    const ID_TP_TRAILER_ARQUIVO = '9';
    
    private $arquivo;

    private $cdagencia;
    private $cdconta;
    private $ocorrenciaHeader;
    private $contHeaderLote = 0;
    /** @var $retorno Retorno */
    private $retorno;

    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco, $nrorg){
        try {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
            $this->nrorg = $nrorg;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getEstruturaArquivo(){
        return array(
            self::ID_TP_ARQUIVO_HEADER_ARQUIVO,
            self::ID_TP_TRANSACAO_1,
            self::ID_TP_TRAILER_ARQUIVO
        );
    }
    
     public function leArquivo(){
        try {
            foreach ($this->arquivo as $linha) {
                $cont = $this->contlinha++;
                if ($linha){
                    $this->retorno = new Retorno();
                    
                    // Tamanho do Arquivo - 240 ou 400
                    $tamanhoLinha = strlen($linha);
                    
                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro =  $idRegistro = $tamanhoLinha == 400?substr($linha, 0, 1):null;
                    
                    if(self::validaArquivo($linha, self::ID_TP_ARQUIVO_HEADER_ARQUIVO, self::getEstruturaArquivo(), $this->contlinha)) {
                        switch ($idRegistro) {
                            case(self::ID_TP_ARQUIVO_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha);
                                break;
                            case(self::ID_TP_TRANSACAO_1):
                                self::leTransacao($linha);
                                break;
                        }
                    }
                    $this->retornos[$cont] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $this->nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    /**
     * @param $cdempresa
     * @throws \Exception
     * @return
     */
    private function getIncricaoEmpresa($cdempresa){
        /** @var $empresa Empresa */
        $empresa = Empresa::findOneByNrorgAndCdempresa($this->nrorg, $cdempresa);
        if($empresa)
            return $empresa->getNrinsjurempr();
        else
            throw new \Exception ('Empresa não Encontrada.');
    }
    
     private function leHeaderArquivo($header){
        // CAMPO: Nº do Daycoval na Câmara Compensação
        // POSIÇÃO: 077 a 079
        // PICTURE: 9(3)
        if(substr($header, 76, 3) != self::CODIGO_COMPENSACAO_DAYCOVAL) {
            self::gravaErros($this->contlinha, 'Número do banco no arquivo é diferente do número do banco informardo no retorno.', true);
        }
        
        // CAMPO: Identificação do Arquivo de Retorno
        // POSIÇÃO: 002 a 002
        // PICTURE: 9(1)
        if (substr($header, 1, 1) != '2'){
            self::gravaErros($this->contlinha, 'Arquivo informado não é um arquivo de retorno', true);
        }
    }
    
    private function leTransacao($detalhe){
        
        // CAMPO: Tipo de Inscrição da empresa
        // POSIÇÃO: 002 a 003
        // PICTURE: 9(2)
        $idtpinscempr = trim(substr($detalhe, 1, 2));
        
        // CAMPO: Inscrição da empresa
        // POSIÇÃO: 004 a 017
        // PICTURE: 9(14)
        $inscjurempr = trim(substr($detalhe, 3, 14));
        
        // CAMPO: Nº controle do participante - chave do boleto
        // POSIÇÃO: 038 a 062
        // PICTURE: X(25)
        $cdfilial = trim(substr($detalhe, 37, 4));
        $nrlanbolrec = trim(substr($detalhe, 41, 10));
        
        // CAMPO: Identificação do Título no Banco
        // POSIÇÃO: 063 a 073
        // PICTURE: 9(11)
        $cdnosnumrec = trim(substr($detalhe, 62, 11));
        
        // CAMPO: Identificação da Ocorrência no Banco
        // POSIÇÃO: 109 a 110
        // PICTURE: 9(2)
        $idocorrencia = trim(substr($detalhe, 108, 2));
        $this->retorno->setIdsitucnabr(self::situacaoCnab($idocorrencia));
        
        // CAMPO: Data da Ocorrencia
        // POSIÇÃO: 111 a 116
        // PICTURE: 9(6)
        $dtOcorrencia = trim(substr($detalhe, 110, 6));
        
        $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);
        $this->retorno->setCdfilial($cdfilial);
        $this->retorno->setNrlanbolrec($nrlanbolrec);
        
        if(empty($idtpinscempr))
            $this->gravaErros($this->contlinha, 'Tipo de inscrição jurídica inválido.', true);
        if(empty($inscjurempr))
            $this->gravaErros($this->contlinha, 'Inscrição jurídica inválida', true);
        if(!empty($cdfilial) && !empty($nrlanbolrec) || !empty($nossonumero)){
            if(empty($idocorrencia))
                $this->gravaErros($this->contlinha, 'Identificação da ocorrência inválido.', true);
            $this->retorno->setBoleto('BOLETO');
            if($boleto){
                // CAMPO: Data de Vencimento do título
                // POSIÇÃO: 147 a 152
                // PICTURE: 9(6)
                $vencimento = $boleto->getDtvenbolrec();
                $this->retorno->setDtvencimento($vencimento->format('d/m/Y'));
                $this->retorno->setCdreducont($boleto->getCdreducont());
                $this->retorno->setCdtpbaixarec($boleto->getCdtpbaixarec());
                $this->retorno->setVrfinalbrec($boleto->getVrfinalbrec());
                $this->retorno->setIdstatrec($boleto->getIdstatbrec());
            }
            
            $empresa = Empresa::findOneByNrinsjurempr($inscjurempr);
            if($empresa)
                $this->retorno->setNmrazsocclie($empresa->getNmrazsocempr());
            
            // CAMPO: Data de vencimento do título
            // POSIÇÃO: 147 a 152
            // PICTURE: 9(13)
            $vencimento = substr($detalhe, 146, 2) . substr($detalhe, 148, 2) . substr(date("Y"), 0, 2) . substr($detalhe, 150, 2);
            $this->retorno->setDtvencimento($vencimento);
            
            // CAMPO: Valor do Título
            // POSIÇÃO: 153 a 165
            // PICTURE: 9(13)
            $this->retorno->setVrnominal(round(floatval(substr($detalhe, 152, 13)) / 100, 2));
            
            $this->retorno->setVrdesconto(
                // CAMPO: IOF Devido
                // POSIÇÃO: 215 a 227
                // PICTURE: 9(13)
                round(floatval(substr($detalhe, 214, 13)) / 100, 2) +
                
                // CAMPO: Desconto concedido
                // POSIÇÃO: 241 a 253
                // PICTURE: 9(13)
                round(floatval(substr($detalhe, 240, 13)) / 100, 2)
                );
            
            $this->retorno->setVracrescimo(
                // CAMPO: Juros operação em atraso
                // POSIÇÃO: 267 a 279
                // PICTURE: 9(13)
                round(floatval(substr($detalhe, 266, 13)) / 100, 2) 

            );
            
            // CAMPO: Valor pago
            // POSIÇÃO: 254 a 266
            // PICTURE: 9(13)
            $this->retorno->setVrpago(round(floatval(substr($detalhe, 253, 13)) / 100, 2));
            $this->retorno->setCdlinha(str_pad($this->contlinha, 4, '0', STR_PAD_LEFT));
            
            // CAMPO: Motivos das rejeições para os códigos de ocorrência
            // POSIÇÃO: 378 a 385
            // PICTURE: 9(8)
            $iderros = trim(substr($detalhe, 377, 8));
            $situacaoCnab = self::situacaoCnab($idocorrencia);
            
            if($situacaoCnab == '01'){
                $primeiroPar = substr($iderros, 0);
                $this->retorno->setOcorrencia($primeiroPar);
            } else if($situacaoCnab == '90'){
                $vencimento = substr($detalhe, 146, 2) . substr($detalhe, 148, 2) . substr(date("Y"), 0, 2) . substr($detalhe, 150, 2);
                $this->retorno->setDtpagamento($vencimento);
            }
        }
        
    }
    
     /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        if($cdocorrencia == '02' or $cdocorrencia == '06' or $cdocorrencia == '08' or $cdocorrencia == '43')
            return '20'; //Retorno OK
        else if($cdocorrencia == '03' or $cdocorrencia == '15' or $cdocorrencia == '16')
            return '01'; //Retorno com Erro
        else if($cdocorrencia == '09' or $cdocorrencia == '10' or $cdocorrencia == '43')
            return '90'; //Baixado
        return '10'; //Sem retorno
    }
}