<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services\Retorno;

use Financeiro\Entities\Ajbolrec;
use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\GeralRec;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Banco\LeitorRetornoSantander;
use Financeiro\Entities\Errcnabr;
use Geral\Entities\Contcorr;
use Geral\Entities\Banco;

abstract class LeitorRetorno {

    const CODIGO_COMPENSACAO_SANTANDER = "033";
    const CODIGO_COMPENSACAO_BRADESCO  = "237";
    const CODIGO_COMPENSACAO_ITAU      = "341";
    const CODIGO_COMPENSACAO_BRASIL    = "001";
    const CODIGO_COMPENSACAO_CAIXA     = "104";
    const CODIGO_COMPENSACAO_SICOOB    = "756";
    const CODIGO_COMPENSACAO_SICREDI   = "748";
    const CODIGO_COMPENSACAO_VOTORANTIM   = "655";
    const CODIGO_COMPENSACAO_BANRISUL   = "041";

    protected $erros = array();
    protected $ocorrencias = array();
    protected $contlinha = 0;
    protected $cdempresa;
    protected $cdbanco;
    protected $retornos = array();

    PROTECTED $arquivoValido;

    public function getErros()
    {
        return $this->erros;
    }

    public function getRetornos(){
        $boletos = array();
        $boletos = array_filter($this->retornos, function($item){
            return strtoupper($item->getBoleto()) == 'BOLETO';
        });
        $ocorrencias = array();
        foreach ($boletos as $boleto) {
            $ocorrencias[] = array(
                'CDLINHA'        => str_pad($boleto->getCdlinha(),4,'0',STR_PAD_LEFT),
                'DTVENCIMENTO'   => $boleto->getDtvencimento(),
                'VRNOMINAL'      => $boleto->getVrnominal(),
                'IDSITUCNABR'    => $boleto->getIdsitucnabr(),
                'VRACRESCIMO'    => $boleto->getVracrescimo(),
                'VRDESCONTO'     => $boleto->getVrdesconto(),
                'DTPAGAMENTO'    => $boleto->getDtpagamento(),
                'VRPAGO'         => $boleto->getVrpago(),
                'NOSSONUMERO'    => $boleto->getNossonumero(),
                'NRLANBOLREC'    => $boleto->getNrlanbolrec(),
                'NMRAZSOCCLIE'   => $boleto->getNmrazsocclie(),
                'VRFINALBREC'    => $boleto->getVrfinalbrec(),
                'CDTPBAIXAREC'   => $boleto->getCdtpbaixarec(),
                'CDREDUCONT'     => $boleto->getCdreducont(),
                'CDFILIAL'       => $boleto->getCdfilial(),
                'DTOCORRENCIA'   => $boleto->getDtocorrencia(),
                'IDNOVACARTEIRA' => $boleto->getIdnovacarteira(),
                'IDCREDBANC'     => $boleto->getIdcredbanc(),
                'VRTXCART'       => $boleto->getValorTaxaCarteira(),
                'IDPROTESTREC'   => $boleto->getIdprotestrec(),
                'CDCARTCOBBOL'   => $boleto->getCarteira(),
                'IDCARTCOBREC'   => $boleto->getIdcarteiracobranca(),
                'SITUACAOBOLETO' => $boleto->getSituacaoOk()
            );
        }
        return $ocorrencias;
    }

    protected function setStatusArquivo($status){
        $this->arquivoValido = $status;
    }

    public function getStatusArquivo(){
        return $this->arquivoValido;
    }

    public abstract function leArquivo();

    protected function gravaErros($cdlinha, $erro, $geral = false){
        $this->erros[] = array(
            'CDLINHA' => str_pad($cdlinha, 4, '0', STR_PAD_LEFT),
            'CDERRO'  => '',
            'DSERRO'  => $erro,
            'GERAL'   => $geral
        );
    }

    protected function validaArquivo($linha, $headerArq, $estruturaArquivo, $contador){

        self::setStatusArquivo(true);

        $idReg = strlen($linha) == 400?substr($linha, 0, 1):substr($linha, 7, 1);
        if($contador == 1 and $idReg != $headerArq){
            self::gravaErros($contador, "Erro de identificação do registro header.",true);
            self::setStatusArquivo(false);
        }
        if($contador > 1 and (! in_array($idReg, $estruturaArquivo)
            )){
            self::gravaErros($contador,"Identificação do registro não é válida",true);
            self::setStatusArquivo(false);
        }

        return self::getStatusArquivo();
    }

    public function preparaOcorrencias($cdbanco, $nrorg){
        /** @var $retorno Retorno */
        foreach($this->retornos as $retorno){

            $ocorrencia = $retorno->getOcorrencia();
            $tamOcorrencia = strlen($retorno->getOcorrencia());
            
            if($cdbanco == '341'){
                $tamanho = 6;
            }elseif($cdbanco == '237'){
                $tamanho = 4;
            }else{
                $tamanho = 2;
            }
            
            for($i = 0; $i<$tamOcorrencia; $i+=$tamanho){

                $cderro = str_pad(substr($ocorrencia, $i, $tamanho), $tamanho);
                
                if($cderro !== "  " && $cderro != '06' && $cderro != '17' && $cderro != '09'){

                    /** @var $errcnabr Errcnabr */
                    $errcnabr = Errcnabr::findOneByCdbancoAndCderrcnabrAndNrorg($cdbanco, $cderro, $nrorg);

                    $dserro = $cderro;
                    if($errcnabr)
                        $dserro.= " - {$errcnabr->getDserrcnabr()}";

                    $this->erros[] = array(
                        'CDLINHA' => $retorno->getCdlinha(),
                        'CDERRO'  => $cderro,
                        'DSERRO'  => $dserro,
                        'BOLETO'  => $retorno->getNrlanbolrec()
                    );
                }
            }
        }
    }

    protected function getTipoBaixaPagamento($cdfilial, $nrlanbolrec){

        /** @var $boletreceber Boletreceber */
        $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);

        if($boletreceber and $boletreceber->getCdtpbaixarec()){
            return $boletreceber->getCdtpbaixarec();
        }
    }

    protected function getReducont($cdfilial, $nrlanbolrec){
        /** @var $boletreceber Boletreceber */
        $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilial($nrlanbolrec, $cdfilial);

        if($boletreceber and $boletreceber->getCdreducont()){
            return $boletreceber->getCdreducont();
        }
    }

    private function getBanco(){
        $banco = Banco::findOneByNrorgAndCdbanco($this->nrorg, $this->cdbanco);
        return $banco;
    }

    public function validaBoletos(){

        /** @var $retorno Retorno  */
        foreach($this->retornos as &$retorno){

            //Se for um boleto
            if($retorno->getBoleto()){

                $retorno->setSituacaoOk(true);
                if(!$retorno->getCdfilial() or !$retorno->getNrlanbolrec()){
                    self::gravaErros($retorno->getCdlinha(),'Código do título não informado no arquivo de retorno.');
                    $retorno->setSituacaoOk(false);
                }
                else{
                    /** @var $boletreceber Boletreceber */
                    $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($retorno->getNrlanbolrec(),
                        $retorno->getCdfilial(),
                        $this->nrorg);
                    $this->validaEstadoTituloReceber($boletreceber, $retorno);

                    $vrpago = round($retorno->getVrpago(), 2);
                    if($boletreceber && $retorno->getIdsitucnabr() == '90'){
                        $vrfinal = round($retorno->getVrnominal() + $retorno->getVracrescimo() - $retorno->getVrdesconto(), 2);
                        if($vrpago != $vrfinal){
                            self::gravaErros($retorno->getCdlinha(), 'Valor final do título está diferente do valor cadastrado.');
                            $retorno->setSituacaoOk(false);
                        }
                    }
                    $this->validaAjusteTitulos($retorno);
                    if($boletreceber){
                        if(! $boletreceber->getCdreducont()){
                            self::gravaErros($retorno->getCdlinha(),'Título não possui conta corrente cadastrada.');
                            $retorno->setSituacaoOk(false);
                        }else{
                            self::validaContaCorrente($retorno, $boletreceber);
                        }
                    }

                    if ($boletreceber) {
                        if (!$boletreceber->getCdtpbaixarec()) {
                            self::gravaErros(
                                $retorno->getCdlinha(), 'Título não possui tipo de baixa a receber cadastrado.'
                            );
                            $retorno->setSituacaoOk(false);
                        }
                    }
                }
            }
        }
    }

    public function validaContaCorrente($retorno, $boletreceber){
        $contasVerificadas = array();
        if($retorno->getCdagencia() and $retorno->getCdconta()){
            
            $contacorrente = Contcorr::findOneByCdreducontAndNrorg($boletreceber->getCdreducont(), $boletreceber->getNrorg());
            
            if ($contacorrente->getCdagencia() && $contacorrente->getCdcontcorr()){
                
                /* Verificação de existência de DÍGITO */
                $banco = Banco::findOneByCdbancoAndNrorg($contacorrente->getCdbanco(), $contacorrente->getNrorg());
                $digitoAgencia   = (strpos($banco->getCdpictagen(), "-") > -1)?true:false;
                $digitoCcorrente = (strpos($banco->getCdpictcont(), "-") > -1)?true:false;
                
                if($digitoCcorrente){ 
                	$contaBoleto = str_pad(substr(trim($contacorrente->getCdcontcorr()), 0, -1), 12, "0", STR_PAD_LEFT);
                	$contaArquivo = str_pad(substr(trim($retorno->getCdconta()), 0, -1), 12, "0", STR_PAD_LEFT);
                }else{ 
                	$contaBoleto = str_pad(substr(trim($contacorrente->getCdcontcorr()), 0, 12), 12, "0", STR_PAD_LEFT);
                	$contaArquivo = str_pad(trim($retorno->getCdconta()), 12, "0", STR_PAD_LEFT);
                }
                
                if($digitoAgencia){
                	$agenciaBoleto = str_pad(substr(trim($contacorrente->getCdagencia()), 0, -1), 5, "0", STR_PAD_LEFT);
                	$agenciaArquivo = str_pad(substr(trim($retorno->getCdagencia()), 0, -1), 5, "0", STR_PAD_LEFT);
                }else{
                	$agenciaBoleto = str_pad(substr(trim($contacorrente->getCdagencia()), 0, 4), 4, "0", STR_PAD_LEFT);
                	$agenciaArquivo = str_pad(substr(trim($retorno->getCdagencia()), 0, 4), 4, "0", STR_PAD_LEFT);
                }
                
                if ($agenciaBoleto != $agenciaArquivo   ||
                    $contaBoleto != $contaArquivo){
                    self::gravaErros($retorno->getCdlinha(),
                                     "Conta corrente de débito {$this->cdbanco} /
                                     {$agenciaArquivo} / {$contaArquivo}
                                     é diferente da conta corrente do título {$this->cdbanco} /
                                     {$agenciaBoleto} / {$contaBoleto}.");
                    $retorno->setSituacaoOk(false);
                }
            }
        }
    }

    public function validaEstadoTituloReceber($boletreceber, $retorno){
        if(!$boletreceber){
            self::gravaErros($retorno->getCdlinha(),'Título não encontrado no sistema.');
            $retorno->setSituacaoOk(false);
        }
        else{
            if($boletreceber->getNrbaixarec()){
                self::gravaErros($retorno->getCdlinha(),'Título encontra-se baixado.');
                $retorno->setSituacaoOk(false);
            }
            if($this->cdempresa != $boletreceber->getCdempresa()){
                self::gravaErros($retorno->getCdlinha(), 'Empresa do título é diferente da empresa informada.');
                $retorno->setSituacaoOk(false);
            }
        }
    }

    public function validaAjusteTitulos($retorno){
        if(round($retorno->getVracrescimo(),2)>0){
            /** @var $geralrec GeralRec */
            $geralrec = GeralRec::findOneByNrorg($this->nrorg);
            if(! $geralrec->getCdajacrerec()){
                self::gravaErros($retorno->getCdlinha(),
                    'Tipo de ajuste não está parametrizado.');
                $retorno->setSituacaoOk(false);
            }
            else{
                $ajbolrec = Ajbolrec::findOneByCdfilialAndNrlanbolrecAndCdajuctrecAndNrorg(
                    $retorno->getCdfilial(),
                    $retorno->getNrlanbolrec(),
                    $geralrec->getCdajacrerec(),
                    $this->nrorg
                );
                if($ajbolrec){
                    self::gravaErros($retorno->getCdlinha(),'Título já possui ajuste cadastrado para o
                                                             tipo de ajuste parametrizado');
                    $retorno->setSituacaoOk(false);
                }
            }
        }
        if(round($retorno->getVrdesconto(),2)>0){
            /** @var $geralrec GeralRec */
            $geralrec = GeralRec::findOneByNrorg($this->nrorg);
            if(! $geralrec->getCdajdescrec()){
                self::gravaErros($retorno->getCdlinha(),
                    'Tipo de ajuste não está parametrizado .');
                $retorno->setSituacaoOk(false);
            }
            else{
                $ajbolrec = Ajbolrec::findOneByCdfilialAndNrlanbolrecAndCdajuctrecAndNrorg(
                    $retorno->getCdfilial(),
                    $retorno->getNrlanbolrec(),
                    $geralrec->getCdajdescrec(),
                    $this->nrorg
                );
                if($ajbolrec){
                    self::gravaErros($retorno->getCdlinha(),'Título já possui ajuste cadastrado para o
                                                             tipo de ajuste parametrizado.');
                    $retorno->setSituacaoOk(false);
                }
            }
        }
    }
}