<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Movreal;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Bco54200ExtratoBancario {
	protected $connection;
	protected $entityManager;

	public function __construct($entityManager) {
		$this->connection = $entityManager->getConnection();
		$this->entityManager = $entityManager;
	}

    /**
     * Busca Movimentação Real por Data, Documento e Valor
     * @param $nrorg
     * @param $cdreducont
     * @param $dtmovreal
     * @param $nrdocmovreal
     * @param $vrmovreal
     * @return Movreal
     */
    public function getMovReal($nrorg, $cdreducont, $dtmovreal, $nrdocmovreal, $vrmovreal, $cdfitidextr = null){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdreducont']   = $cdreducont;
        $params['dtmovreal']    = $dtmovreal;
        $params['nrdocmovreal'] = $nrdocmovreal;
        $params['vrmovreal']    = $vrmovreal;
        
        if($cdfitidextr)
        	$params['cdfitidextr']    = $cdfitidextr;

        /** @var Movreal $movreal */
        $movreal = $this->entityManager->getRepository(Movreal::class)->findOneBy($params);
        
        return $movreal;
    }

	public function calculaSaldoPerio($dtinicial, $dtfinal, $cdreducont, $nrorg) {
		$sql="SELECT (NVL(C.CREDITO, 0) - NVL(D.DEBITO, 0)) AS SALDO
				FROM
				  (SELECT SUM(VRMOVREAL) AS DEBITO
				  FROM MOVREAL
				  WHERE IDMOVREAL = 'D'
				  AND DTMOVREAL BETWEEN :DTINICIAL AND :DTFINAL
				  AND CDREDUCONT = :CDREDUCONT
				  AND NRORG = :NRORG
				  ) D,
				  (SELECT SUM(VRMOVREAL) AS CREDITO
				  FROM MOVREAL
				  WHERE IDMOVREAL = 'C'
				  AND DTMOVREAL BETWEEN :DTINICIAL AND :DTFINAL
				  AND CDREDUCONT = :CDREDUCONT
				  AND NRORG = :NRORG
				  ) C ";

		$params = array("DTINICIAL"=>$dtinicial,
						"DTFINAL"=>$dtfinal,
						"CDREDUCONT"=>$cdreducont,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}

	public function geraLog($cdreducont, $cdsequextr, $nrseqmovreal, $cdoperador, $dtaltmovreal, $nrorg, $idativo,
	  $dtultatu, $nrorgultatu, $cdoperultatu, $dtinclusao, $nrorginclusao, $cdoperinclusao) {

		$sql = "INSERT INTO LOGMOVREAL
				   (CDREDUCONT, CDSEQUEXTR, NRSEQMOVREAL, CDOPERADOR, DTALTMOVREAL , NRORG, IDATIVO, DTULTATU,
					NRORGULTATU, CDOPERULTATU, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO)
				 VALUES
				   (:CDREDUCONT, :CDSEQUEXTR, :NRSEQMOVREAL, :CDOPERADOR, :DTALTMOVREAL, :NRORG, :IDATIVO,
				   :DTULTATU, :NRORGULTATU, :CDOPERULTATU, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO)";

		$params = array("CDREDUCONT"=>$cdreducont,
						"CDSEQUEXTR"=>$cdsequextr,
						"NRSEQMOVREAL"=>$nrseqmovreal,
						"CDOPERADOR"=>$cdoperador,
						"DTALTMOVREAL"=>$dtaltmovreal,
						"NRORG"=>$nrorg,
						"IDATIVO"=>$idativo,
						"DTULTATU"=>$dtultatu,
						"NRORGULTATU"=>$nrorgultatu,
						"CDOPERULTATU"=>$cdoperultatu,
						"DTINCLUSAO"=>$dtinclusao,
						"NRORGINCLUSAO"=>$nrorginclusao,
						"CDOPERINCLUSAO"=>$cdoperinclusao);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

	public function getBancoByCdccnab($cdbanccnab) {
		$sql = "SELECT NMBANCO, CDBANCO
				FROM BANCO
				WHERE CDBANCCNAB = :CDBANCCNAB";

		$params = array("CDBANCCNAB"=>$cdbanccnab);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetchAll();
	}
	public function getCdpictcont($cdbanco, $nrorg) {
		$sql="SELECT CDPICTCONT
				FROM BANCO
				WHERE ((NRORG = :NRORG) OR (NRORG = 0))
				AND CDBANCCNAB = :CDBANCO";

		$params = array("CDBANCO"=>$cdbanco,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}

	public function getContas($cdcontcorr, $cdbanco, $nrorg, $empresa) {
		$sql="SELECT CC.CDREDUCONT,
				       CC.CDBANCO,
				       CC.CDAGENCIA,
				       CC.CDCONTCORR,
				       B.CDPICTCONT,
				       CC.IDTIPOCCORR,
				       CC.CDEMPRESA,
				       CC.DSCONTCORR,
				       B.NMBANCO
				       FROM CONTCORR  CC, BANCO B
				 WHERE (CC.CDBANCO = B.CDBANCO (+))
				   AND LPAD(B.CDBANCCNAB,4,0) = LPAD(:CDBANCO,4,0)
				   AND trim((LPAD(SUBSTR(CC.CDCONTCORR, 1, INSTR(B.CDPICTCONT, '-', 1, 1)-1) , 20,'0'))) = trim(LPAD(:CDCONTCORR , 20,'0'))
				   AND CC.NRORG = :NRORG
				   AND (:CDEMPRESA = 'T' OR INSTR(:CDEMPRESA, CC.CDEMPRESA) <> 0)
				 UNION
				SELECT CC.CDREDUCONT,
				       CC.CDBANCO,
				       CC.CDAGENCIA,
				       CC.CDCONTCORR,
				       B.CDPICTCONT,
				       CC.IDTIPOCCORR,
				       CC.CDEMPRESA,
				       CC.DSCONTCORR,
				       B.NMBANCO
				  FROM CONTCORR  CC, BANCO B
				 WHERE (CC.CDBANCO = B.CDBANCO (+))
				   AND LPAD(B.CDBANCCNAB,4,0) = LPAD(:CDBANCO,4,0)
				   AND trim((LPAD(CC.CDCONTCORR , 20,'0'))) = trim(LPAD(:CDCONTCORR , 20,'0'))
				   AND CC.NRORG  = :NRORG
				   AND (:CDEMPRESA = 'T' OR INSTR(:CDEMPRESA, CC.CDEMPRESA) <> 0)
				 UNION
				SELECT C.CDREDUCONT,
				       C.CDBANCO,
				       C.CDAGENCIA,
				       C.CDCONTCORR,
				       B.CDPICTCONT,
				       C.IDTIPOCCORR,
				       C.CDEMPRESA,
				       C.DSCONTCORR,
				       B.NMBANCO
				  FROM CONTCORR C, BANCO B
				 WHERE (LPAD(C.CDCONTCORR,20,0) = LPAD(:CDCONTCORR,20,0) or
				        LPAD(C.CDAGENCIA||C.CDCONTCORR,25,0) = LPAD(:CDCONTCORR,25,0) OR
				        LPAD(SUBSTR(C.CDAGENCIA, 1, 4)||C.CDCONTCORR,25,0) = LPAD(:CDCONTCORR,25,0))
				   AND LPAD(B.CDBANCCNAB,4,0) = LPAD(:CDBANCO,4,0)
				   AND C.CDBANCO = B.CDBANCO
				   AND C.NRORG   = :NRORG
				   AND (:CDEMPRESA = 'T' OR INSTR(:CDEMPRESA, C.CDEMPRESA) <> 0)
				 UNION
				SELECT C.CDREDUCONT,
				       C.CDBANCO,
				       C.CDAGENCIA,
				       C.CDCONTCORR,
				       B.CDPICTCONT,
				       C.IDTIPOCCORR,
				       C.CDEMPRESA,
				       C.DSCONTCORR,
				       B.NMBANCO
				  FROM CONTCORR C, BANCO B
				 WHERE (LPAD(C.CDCONTCORR,20,0) = LPAD(:CDCONTCORR,20,0) or
				       LPAD(C.CDAGENCIA||C.CDCONTCORR,25,0) = LPAD(:CDCONTCORR,25,0) OR
				       LPAD(SUBSTR(C.CDAGENCIA, 1, length(B.CDPICTAGEN))||C.CDCONTCORR,25,0) = LPAD(:CDCONTCORR,25,0) OR
				       LPAD(SUBSTR(C.CDAGENCIA, 1, length(B.CDPICTAGEN))||C.CDCONTCORR,25,0) = LPAD(SUBSTR(:CDCONTCORR,1,length(B.CDPICTAGEN))||SUBSTR(:CDCONTCORR,-LENGTH(B.CDPICTCONT)),25,0))
				   AND LPAD(B.CDBANCCNAB,4,0) = LPAD(:CDBANCO,4,0)
				   AND C.CDBANCO = B.CDBANCO
				   AND C.NRORG   = :NRORG
				   AND (:CDEMPRESA = 'T' OR INSTR(:CDEMPRESA, C.CDEMPRESA) <> 0)
				 ORDER BY IDTIPOCCORR";

		$params = array("CDCONTCORR"=>$cdcontcorr,
						"CDBANCO"=>$cdbanco,
						"NRORG"=>$nrorg,
						"CDEMPRESA" => $empresa);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetchAll();
	}

	Public function getContasBrasil($cdcontcorr, $cdbanco, $nrorg) {
		$sql="SELECT C.CDREDUCONT,
					  C.CDBANCO,
					  C.CDAGENCIA,
					  C.CDCONTCORR,
					  C.DSCONTCORR,
					  B.CDPICTCONT,
					  C.IDTIPOCCORR,
					  B.NMBANCO
					FROM CONTCORR C,
					  BANCO B
					WHERE (LPAD(C.CDCONTCORR, 20, 0) = LPAD(:CDCONTCORR, 20, 0)
					OR LPAD(C.CDAGENCIA
					  ||C.CDCONTCORR, 25, 0) = LPAD(:CDCONTCORR, 25, 0)
					OR LPAD(SUBSTR(C.CDAGENCIA, 1, 4)
					  ||C.CDCONTCORR, 25, 0)     = LPAD(:CDCONTCORR, 25, 0))
					AND LPAD(B.CDBANCCNAB, 4, 0) = LPAD(:CDBANCO, 4, 0)
					AND C.CDBANCO                = B.CDBANCO
					AND C.NRORG = :NRORG";

		$params = array("CDCONTCORR"=>$cdcontcorr,
						"CDBANCO"=>$cdbanco,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetchAll();
	}

	public function getContCorrLike($cdbanco, $cdagencia, $cdcontcorr, $nrorg) {
		$sql="SELECT CC.CDREDUCONT, CC.DSCONTCORR
				FROM CONTCORR CC,
				   BANCO    BC
				WHERE (BC.CDBANCCNAB = :CDBANCO)
				   AND (CC.CDBANCO    = BC.CDBANCO)
				   AND (CC.CDAGENCIA like :CDAGENCIA || '%')
				   AND (CC.CDCONTCORR  like :CDCONTCORR || '%')
				   AND CC.NRORG = :NRORG";

		$params = array("CDBANCO"=>$cdbanco,
						"CDAGENCIA"=>$cdagencia,
						"CDCONTCORR"=>$cdcontcorr,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}

	public function getExtratoMes($primeiroDia, $ultimoDia, $cdreducont, $nrorg) {
		$sql="SELECT VRSALDINIC, IDEXTRINI, DTSALDINIC, VRSALDFIM, IDEXTRFIM, DTSALDFIN, VRSALDINIC, IDEXTRFIM, DTSALDINIC, CDSEQUEXTR
				FROM EXTRBANCO
				WHERE DTSALDINIC = :DTINICIAL
				  AND DTSALDFIN  = :DTFINAL
				  AND CDREDUCONT = :CDREDUCONT
				  AND NRORG = :NRORG";
		$params = array("DTINICIAL"=>$primeiroDia,
						"DTFINAL"=>$ultimoDia,
						"CDREDUCONT"=>$cdreducont,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}
	
	public function getExtratoCdsequextr($cdsequextr, $cdreducont, $nrorg) {
		$sql="SELECT CDREDUCONT, CDSEQUEXTR, VRSALDINIC, IDEXTRINI, DTSALDINIC, VRSALDFIM, IDEXTRFIM, DTSALDFIN
				FROM EXTRBANCO
				WHERE CDREDUCONT = :CDREDUCONT
                  AND CDSEQUEXTR = :CDSEQUEXTR
                  AND NRORG = :NRORG";
		$params = array("CDSEQUEXTR"=>$cdsequextr,
						"CDREDUCONT"=>$cdreducont,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}

	public function getMovsPeriodo($dtinicial, $dtfinal, $cdreducont, $nrorg) {
		$sql="SELECT COUNT(CDSEQUEXTR) LINHAS
				FROM MOVREAL
				WHERE DTMOVREAL BETWEEN :DTINICIAL AND :DTFINAL
				  AND CDREDUCONT = :CDREDUCONT
				  AND NRORG = :NRORG";

		$params = array("DTINICIAL"=>$dtinicial,
						"DTFINAL"=>$dtfinal,
						"CDREDUCONT"=>$cdreducont,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}

	public function getNrseqmovreal($cdreducont, $cdsequextr, $nrorg) {
		$sql = "SELECT
				  MAX(M.NRSEQMOVREAL) NRSEQMOVREAL
				FROM
				  MOVREAL M
				WHERE M.CDREDUCONT = :CDREDUCONT
				  AND M.CDSEQUEXTR = :CDSEQUEXTR
				  AND M.NRORG = :NRORG";

		$params = array("CDREDUCONT"=>$cdreducont,
						"CDSEQUEXTR"=>$cdsequextr,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}

	public function getPesqLanc($cdreducont, $nrdocmovreal, $dtmovreal, $vrmovreal, $nrorg) {
		$sql="SELECT CDREDUCONT,
			   NRDOCMOVREAL,
			   DTMOVREAL,
			   VRMOVREAL
			FROM MOVREAL
			WHERE (CDREDUCONT   = :CDREDUCONT)
			   AND (LPAD(NRDOCMOVREAL, 20, 0) = LPAD(:NRDOCMOVREAL, 20, 0))
			   AND (DTMOVREAL    = :DTMOVREAL)
			   AND (VRMOVREAL    = :VRMOVREAL)
			   AND NRORG = :NRORG
			ORDER BY CDREDUCONT,
			   NRDOCMOVREAL,
			   DTMOVREAL,
			   VRMOVREAL";

		$params = array("CDREDUCONT" => $cdreducont,
						"NRDOCMOVREAL" => $nrdocmovreal,
						"DTMOVREAL" => $dtmovreal,
						"VRMOVREAL" => $vrmovreal,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetchAll();
	}

	public function getSaldoInicRecente($data, $cdreducont, $nrorg) {
		$sql="SELECT IDEXTRINI,
				  VRSALDINIC,
				  DTSALDINIC
				FROM EXTRBANCO
				WHERE DTSALDINIC < :DATA
				  AND CDREDUCONT   = :CDREDUCONT
				  AND NRORG        = :NRORG
				ORDER BY TO_CHAR(DTSALDINIC, 'YYYYMMDD') DESC";

		$params = array("DATA"=>$data,
						"CDREDUCONT"=>$cdreducont,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}

	public function insertExtrBanco(
		$cdreducont, $cdsequextr, $dtsaldinic, $dtsaldfin, $vrsaldinic, $vrsaldfim,
		$idextrini, $idextrfim, $nrorg, $idativo, $dtultatu, $nrorgultatu, $cdoperultatu,
		$dtinclusao, $nrorginclusao, $cdoperinclusao) {

		$sql = "INSERT INTO EXTRBANCO
				  (CDREDUCONT, CDSEQUEXTR, DTSALDINIC, DTSALDFIN, VRSALDINIC, VRSALDFIM,
				  IDEXTRINI, IDEXTRFIM, NRORG, IDATIVO, DTULTATU, NRORGULTATU, CDOPERULTATU,
				  DTINCLUSAO,NRORGINCLUSAO, CDOPERINCLUSAO)
				VALUES
				  (:CDREDUCONT, :CDSEQUEXTR, :DTSALDINIC, :DTSALDFIN, :VRSALDINIC, :VRSALDFIM,
					:IDEXTRINI, :IDEXTRFIM, :NRORG, :IDATIVO, :DTULTATU, :NRORGULTATU,
				  :CDOPERULTATU, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO)";

		$params = array("CDREDUCONT"=>$cdreducont,
						"CDSEQUEXTR"=>$cdsequextr,
						"DTSALDINIC"=>$dtsaldinic,
						"DTSALDFIN"=>$dtsaldfin,
						"VRSALDINIC"=>$vrsaldinic,
						"VRSALDFIM"=>$vrsaldfim,
						"IDEXTRINI"=>$idextrini,
						"IDEXTRFIM"=>$idextrfim,
						"NRORG"=>$nrorg,
						"IDATIVO"=>$idativo,
						"DTULTATU"=>$dtultatu,
						"NRORGULTATU"=>$nrorgultatu,
						"CDOPERULTATU"=>$cdoperultatu,
						"DTINCLUSAO"=>$dtinclusao,
						"NRORGINCLUSAO"=>$nrorginclusao,
						"CDOPERINCLUSAO"=>$cdoperinclusao);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

	public function insertMovReal($cdreducont, $cdsequextr, $nrseqmovreal, $nrdocmovreal, $dtmovreal, $vrmovreal,
		$idmovreal, $dshistmovreal, $idstconmvban, $idtiporegi, $cdcatelanc, $idconcreal, $idtpmovreal,
		$nrorg, $idativo, $dtultatu, $nrorgultatu, $cdoperultatu, $dtinclusao, $nrorginclusao, $cdoperinclusao, $cdfitidextr = null) {
		$sql = "INSERT INTO MOVREAL
				  (CDREDUCONT, CDSEQUEXTR, NRSEQMOVREAL, NRDOCMOVREAL, DTMOVREAL, VRMOVREAL,
				   IDMOVREAL, DSHISTMOVREAL, IDSTCONMVBAN, IDTIPOREGI, CDCATELANC, IDCONCREAL,
				   IDTPMOVREAL, NRORG, IDATIVO, DTULTATU, NRORGULTATU, CDOPERULTATU, DTINCLUSAO,
				   NRORGINCLUSAO, CDOPERINCLUSAO, CDFITIDEXTR)
				values
				  (:CDREDUCONT, :CDSEQUEXTR, :NRSEQMOVREAL, :NRDOCMOVREAL, :DTMOVREAL,
				   :VRMOVREAL, :IDMOVREAL, :DSHISTMOVREAL, :IDSTCONMVBAN, :IDTIPOREGI,
				   :CDCATELANC, :IDCONCREAL, :IDTPMOVREAL, :NRORG, :IDATIVO, :DTULTATU,
				   :NRORGULTATU, :CDOPERULTATU, :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO, :CDFITIDEXTR)";

		$params = array("CDREDUCONT"=>$cdreducont,
						"CDSEQUEXTR"=>$cdsequextr,
						"NRSEQMOVREAL"=>$nrseqmovreal,
						"NRDOCMOVREAL"=>$nrdocmovreal,
						"DTMOVREAL"=>$dtmovreal,
						"VRMOVREAL"=>$vrmovreal,
						"IDMOVREAL"=>$idmovreal,
						"DSHISTMOVREAL"=>$dshistmovreal,
						"IDSTCONMVBAN"=>$idstconmvban,
						"IDTIPOREGI"=>$idtiporegi,
						"CDCATELANC"=>$cdcatelanc,
						"IDCONCREAL"=>$idconcreal,
						"IDTPMOVREAL"=>$idtpmovreal,
						"NRORG"=>$nrorg,
						"IDATIVO"=>$idativo,
						"DTULTATU"=>$dtultatu,
						"NRORGULTATU"=>$nrorgultatu,
						"CDOPERULTATU"=>$cdoperultatu,
						"DTINCLUSAO"=>$dtinclusao,
						"NRORGINCLUSAO"=>$nrorginclusao,
						"CDOPERINCLUSAO"=>$cdoperinclusao,
						"CDFITIDEXTR"=>$cdfitidextr);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

	public function registrosExtr($cdreducont, $nrorg) {
		$sql="SELECT COUNT(*) AS REGISTROS
				FROM EXTRBANCO
				WHERE CDREDUCONT = :CDREDUCONT
				  AND NRORG = :NRORG";

		$params = array("CDREDUCONT"=>$cdreducont,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetch();
	}

	public function removeExtrBanco($cdreducont, $cdsequextr, $nrorg) {
		$sql="DELETE FROM EXTRBANCO
			WHERE CDREDUCONT = :CDREDUCONT
			  AND CDSEQUEXTR = :CDSEQUEXTR
			  AND NRORG = :NRORG";
		$params = array("CDREDUCONT"=>$cdreducont,
						"CDSEQUEXTR"=>$cdsequextr,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

	public function removeMovReal($cdreducont, $cdsequextr, $nrseqmovreal, $nrorg) {
		$sql="DELETE FROM MOVREAL
				WHERE CDREDUCONT = :CDREDUCONT
				AND CDSEQUEXTR = :CDSEQUEXTR
				AND NRSEQMOVREAL = :NRSEQMOVREAL
				AND NRORG = :NRORG";
		$params = array("CDREDUCONT"=>$cdreducont,
						"CDSEQUEXTR"=>$cdsequextr,
						"NRSEQMOVREAL"=>$nrseqmovreal,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

	public function updateExtrBanco($cdreducont, $cdsequextr, $vrsaldfim, $idextrfim, $nrorg, $dtultatu, $nrorgultatu, $cdoperultatu) {
		$sql="UPDATE EXTRBANCO SET
				VRSALDFIM = :VRSALDFIM,
				IDEXTRFIM = :IDEXTRFIM,
				DTULTATU = :DTULTATU,
				NRORGULTATU = :NRORGULTATU,
				CDOPERULTATU = :CDOPERULTATU
			WHERE CDREDUCONT = :CDREDUCONT
				AND CDSEQUEXTR = :CDSEQUEXTR
				AND NRORG = :NRORG";

		$params = array("CDREDUCONT"=>$cdreducont,
						"CDSEQUEXTR"=>$cdsequextr,
						"VRSALDFIM"=>$vrsaldfim,
						"IDEXTRFIM"=>$idextrfim,
						"NRORG"=>$nrorg,
						"DTULTATU"=>$dtultatu,
						"NRORGULTATU"=>$nrorgultatu,
						"CDOPERULTATU"=>$cdoperultatu);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

	public function updateMovReal($cdreducont, $cdsequextr, $nrseqmovreal, $nrdocmovreal, $dtmovreal, $vrmovreal,
		$idmovreal, $dshistmovreal, $idstconmvban, $idtiporegi, $cdcatelanc, $idconcreal, $idtpmovreal, $nrorg,
		$idativo, $dtultatu, $nrorgultatu, $cdoperultatu, $cdfitidextr = null) {
		$sql="UPDATE MOVREAL SET
			   NRDOCMOVREAL = :NRDOCMOVREAL,
			   DTMOVREAL = :DTMOVREAL,
			   VRMOVREAL = :VRMOVREAL,
			   IDMOVREAL = :IDMOVREAL,
			   DSHISTMOVREAL = :DSHISTMOVREAL,
			   IDSTCONMVBAN = :IDSTCONMVBAN,
			   IDTIPOREGI = :IDTIPOREGI,
			   CDCATELANC = :CDCATELANC,
			   IDCONCREAL = :IDCONCREAL,
			   IDTPMOVREAL = :IDTPMOVREAL,
			   IDATIVO = :IDATIVO,
			   DTULTATU = :DTULTATU,
			   NRORGULTATU = :NRORGULTATU,
			   CDOPERULTATU = :CDOPERULTATU,
			   CDFITIDEXTR = :CDFITIDEXTR
			 WHERE CDREDUCONT = :CDREDUCONT
			   AND CDSEQUEXTR = :CDSEQUEXTR
			   AND NRSEQMOVREAL = :NRSEQMOVREAL
			   AND NRORG = :NRORG";

			$params = array("CDREDUCONT"=>$cdreducont,
							"CDSEQUEXTR"=>$cdsequextr,
							"NRSEQMOVREAL"=>$nrseqmovreal,
							"NRDOCMOVREAL"=>$nrdocmovreal,
							"DTMOVREAL"=>$dtmovreal,
							"VRMOVREAL"=>$vrmovreal,
							"IDMOVREAL"=>$idmovreal,
							"DSHISTMOVREAL"=>$dshistmovreal,
							"IDSTCONMVBAN"=>$idstconmvban,
							"IDTIPOREGI"=>$idtiporegi,
							"CDCATELANC"=>$cdcatelanc,
							"IDCONCREAL"=>$idconcreal,
							"IDTPMOVREAL"=>$idtpmovreal,
							"CDREDUCONT"=>$cdreducont,
							"CDSEQUEXTR"=>$cdsequextr,
							"NRSEQMOVREAL"=>$nrseqmovreal,
							"NRORG"=>$nrorg,
							"IDATIVO"=>$idativo,
							"DTULTATU"=>$dtultatu,
							"NRORGULTATU"=>$nrorgultatu,
							"CDOPERULTATU"=>$cdoperultatu,
							"CDFITIDEXTR"=>$cdfitidextr);
			$query = $this->connection->prepare($sql);
			$query->execute($params);
		}

	public function validaOperador($cdoperador, $cdreducont, $nrorg) {
		$sql = "SELECT CC.CDREDUCONT
				FROM FILIAL FI, FILIOPER FO, CONTCORR CC
				WHERE FI.NRORG = :NRORG
				  AND FI.CDFILIAL = FO.CDFILIAL
				  AND FI.NRORG = FO.NRORG
				  AND CC.CDEMPRESA = FI.CDEMPRESA
				  AND CC.NRORG = FI.NRORG
				  AND FO.CDOPERADOR = :CDOPERADOR
				  AND CC.CDREDUCONT = :CDREDUCONT
				UNION
				SELECT CC.CDREDUCONT
				FROM EMPRCNTRFIL EF, FILIAL FI, FILIOPER FO, CONTCORR CC
				WHERE FI.NRORG = :NRORG
				  AND FI.CDFILIAL = FO.CDFILIAL
				  AND FI.NRORG = FO.NRORG
				  AND FI.CDFILIAL = EF.CDFILIAL
				  AND FI.NRORG = EF.NRORG
				  AND CC.CDEMPRESA = EF.CDEMPRESA
				  AND CC.NRORG = EF.NRORG
				  AND FO.CDOPERADOR = :CDOPERADOR
				  AND CC.CDREDUCONT = :CDREDUCONT";

		$params = array("CDOPERADOR"=>$cdoperador,
						"CDREDUCONT"=>$cdreducont,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetchAll();
	}
	
	
	public function buscaExtratoImportado($param) {
		
		$sql = "SELECT IDSTCONMVBAN, IDTPMOVREAL 
						FROM MOVREAL
				    WHERE CDREDUCONT = :CDREDUCONT
				    AND CDSEQUEXTR = :CDSEQUEXTR
				    AND NRSEQMOVREAL = :NRSEQMOVREAL";

		$query = $this->connection->prepare($sql);
		$query->execute($param);
		return $query->fetchAll();
	}
	
	public function getExtrato($params) {
		
		$sql = "SELECT 
					  CDREDUCONT, 
					  CDSEQUEXTR, 
					  NRSEQMOVREAL, 
					  NRDOCMOVREAL, 
					  DTMOVREAL, 
					  VRMOVREAL, 
					  IDMOVREAL, 
					  DSHISTMOVREAL, 
					  IDTPMOVREAL,
					  CDFITIDEXTR
					FROM 
					  MOVREAL 
					WHERE (CDREDUCONT = :CDREDUCONT)
					  AND (:CDSEQUEXTR = :CDSEQUEXTR) 
					  AND (
					    LPAD(NRDOCMOVREAL, 20, 0) = LPAD(:NRDOCMOVREAL, 20, 0)
					  )
					  AND (DSHISTMOVREAL = :DSHISTMOVREAL)
					  AND (DTMOVREAL = :DTMOVREAL) 
					  AND (VRMOVREAL = :VRMOVREAL) 
					  AND ( IDMOVREAL= :IDMOVREAL) 
					  AND ( IDTPMOVREAL = :IDTPMOVREAL) 
					  AND ( CDFITIDEXTR = :CDFITIDEXTR) 
					  AND (NRORG = :NRORG)
					ORDER BY 
					  CDREDUCONT, 
					  NRDOCMOVREAL, 
					  DTMOVREAL, 
					  VRMOVREAL";

		$query = $this->connection->prepare($sql);
		$query->execute($params);
		return $query->fetchAll();
	}
}